@@ -40,6 +40,7 @@ Budget::Budget() {
 	securityTransactions.setAutoDelete(true);
 	transactions.setAutoDelete(false);
 	scheduledTransactions.setAutoDelete(true);
+	splitTransactions.setAutoDelete(true);
 	securities.setAutoDelete(true);
 	expensesAccounts.setAutoDelete(true);
 	incomesAccounts.setAutoDelete(true);
@@ -57,6 +58,7 @@ Budget::~Budget() {}
 void Budget::clear() {
 	transactions.clear();
 	scheduledTransactions.clear();
+	splitTransactions.clear();
 	securities.clear();
 	expenses.clear();
 	incomes.clear();
@@ -128,7 +130,7 @@ QString Budget::loadFile(QString filename, QString &errors) {
 			} else if(e.tagName() == "transaction") {
 				QString type = e.attribute("type");
 				bool valid = true;
-				if(type == "expense") {
+				if(type == "expense" || type == "refund") {
 					Expense *expense = new Expense(this, &e, &valid);
 					if(valid) {
 						expenses.append(expense);
@@ -137,7 +139,7 @@ QString Budget::loadFile(QString filename, QString &errors) {
 						transaction_errors++;
 						delete expense;
 					}
-				} else if(type == "income") {
+				} else if(type == "income" || type == "repayment") {
 					Income *income = new Income(this, &e, &valid);
 					if(valid) {
 						incomes.append(income);
@@ -236,6 +238,34 @@ QString Budget::loadFile(QString filename, QString &errors) {
 						transaction_errors++;
 						delete trans;
 					}
+				} else if(type == "split") {
+					SplitTransaction *split = new SplitTransaction(this, &e, &valid);
+					if(valid) {
+						splitTransactions.append(split);
+						QValueVector<Transaction*>::size_type c = split->splits.count();
+						for(QValueVector<Transaction*>::size_type i = 0; i < c; i++) {
+							Transaction *trans = split->splits[i];
+							switch(trans->type()) {
+								case TRANSACTION_TYPE_TRANSFER: {
+									transfers.append((Transfer*) trans);
+									break;
+								}
+								case TRANSACTION_TYPE_INCOME: {
+									incomes.append((Income*) trans);
+									break;
+								}
+								case TRANSACTION_TYPE_EXPENSE: {
+									expenses.append((Expense*) trans);
+									break;
+								}
+								default: {}
+							}
+							transactions.append(trans);
+						}
+					} else {
+						transaction_errors++;
+						delete split;
+					}
 				}
 			} else if(e.tagName() == "category") {
 				QString type = e.attribute("type");
@@ -307,6 +337,7 @@ QString Budget::loadFile(QString filename, QString &errors) {
 	}
 	transactions.sort();
 	scheduledTransactions.sort();
+	splitTransactions.sort();
 
 	expensesAccounts.sort();
 	incomesAccounts.sort();
@@ -335,7 +366,7 @@ QString Budget::loadFile(QString filename, QString &errors) {
 		errors += i18n("Unable to load 1 transaction.", "Unable to load %n transactions.", transaction_errors);
 		had_line = true;
 	}
-
+	file.close();
 	return QString::null;
 }
 
@@ -421,6 +452,15 @@ QString Budget::saveFile(QString filename) {
 		strans = scheduledTransactions.next();
 	}
 
+	SplitTransaction *split = splitTransactions.first();
+	while(split) {
+		QDomElement e = doc.createElement("transaction");
+		e.setAttribute("type", "split");
+		split->save(&e);
+		root.appendChild(e);
+		split = splitTransactions.next();
+	}
+
 	security = securities.first();
 	while(security) {
 		ReinvestedDividend *rediv = security->reinvestedDividends.first();
@@ -452,33 +492,37 @@ QString Budget::saveFile(QString filename) {
 
 	Transaction *trans = transactions.first();
 	while(trans) {
-		QDomElement e = doc.createElement("transaction");
-		switch(trans->type()) {
-			case TRANSACTION_TYPE_TRANSFER: {
-				if(trans->fromAccount() == balancingAccount || trans->toAccount() == balancingAccount) e.setAttribute("type", "balancing");
-				else e.setAttribute("type", "transfer");
-				break;
-			}
-			case TRANSACTION_TYPE_INCOME: {
-				if(((Income*) trans)->security()) e.setAttribute("type", "dividend");
-				else e.setAttribute("type", "income");
-				break;
-			}
-			case TRANSACTION_TYPE_EXPENSE: {
-				e.setAttribute("type", "expense");
-				break;
-			}
-			case TRANSACTION_TYPE_SECURITY_BUY: {
-				e.setAttribute("type", "security_buy");
-				break;
-			}
-			case TRANSACTION_TYPE_SECURITY_SELL: {
-				e.setAttribute("type", "security_sell");
-				break;
+		if(!trans->parentSplit()) {
+			QDomElement e = doc.createElement("transaction");
+			switch(trans->type()) {
+				case TRANSACTION_TYPE_TRANSFER: {
+					if(trans->fromAccount() == balancingAccount || trans->toAccount() == balancingAccount) e.setAttribute("type", "balancing");
+					else e.setAttribute("type", "transfer");
+					break;
+				}
+				case TRANSACTION_TYPE_INCOME: {
+					if(((Income*) trans)->security()) e.setAttribute("type", "dividend");
+					else if(trans->value() < 0.0) e.setAttribute("type", "repayment");
+					else e.setAttribute("type", "income");
+					break;
+				}
+				case TRANSACTION_TYPE_EXPENSE: {
+					if(trans->value() < 0.0) e.setAttribute("type", "refund");
+					else e.setAttribute("type", "expense");
+					break;
+				}
+				case TRANSACTION_TYPE_SECURITY_BUY: {
+					e.setAttribute("type", "security_buy");
+					break;
+				}
+				case TRANSACTION_TYPE_SECURITY_SELL: {
+					e.setAttribute("type", "security_sell");
+					break;
+				}
 			}
+			trans->save(&e);
+			root.appendChild(e);
 		}
-		trans->save(&e);
-		root.appendChild(e);
 		trans = transactions.next();
 	}
 
@@ -521,6 +565,10 @@ void Budget::addTransaction(Transaction *trans) {
 	transactions.inSort(trans);
 }
 void Budget::removeTransaction(Transaction *trans, bool keep) {
+	if(trans->parentSplit()) {
+		trans->parentSplit()->removeTransaction(trans, keep);
+		return;
+	}
 	transactions.removeRef(trans);
 	switch(trans->type()) {
 		case TRANSACTION_TYPE_EXPENSE: {
@@ -556,6 +604,44 @@ void Budget::removeTransaction(Transaction *trans, bool keep) {
 		}
 	}
 }
+void Budget::addSplitTransaction(SplitTransaction *split) {
+	splitTransactions.inSort(split);
+	QValueVector<Transaction*>::size_type c = split->splits.count();
+	for(QValueVector<Transaction*>::size_type i = 0; i < c; i++) {
+		addTransaction(split->splits[i]);
+	}
+}
+void Budget::removeSplitTransaction(SplitTransaction *split, bool keep) {
+	QValueVector<Transaction*>::size_type c = split->splits.count();
+	for(QValueVector<Transaction*>::size_type i = 0; i < c; i++) {
+		Transaction *trans = split->splits[i];
+		transactions.removeRef(trans);
+		switch(trans->type()) {
+			case TRANSACTION_TYPE_EXPENSE: {
+				expenses.setAutoDelete(false);
+				expenses.removeRef((Expense*) trans);
+				expenses.setAutoDelete(true);
+				break;
+			}
+			case TRANSACTION_TYPE_INCOME: {
+				incomes.setAutoDelete(false);
+				incomes.removeRef((Income*) trans);
+				incomes.setAutoDelete(true);
+				break;
+			}
+			case TRANSACTION_TYPE_TRANSFER: {
+				transfers.setAutoDelete(false);
+				transfers.removeRef((Transfer*) trans);
+				transfers.setAutoDelete(true);
+				break;
+			}
+			default: {}
+		}
+	}
+	if(keep) splitTransactions.setAutoDelete(false);
+	splitTransactions.removeRef(split);
+	if(keep) splitTransactions.setAutoDelete(true);
+}
 void Budget::addScheduledTransaction(ScheduledTransaction *strans) {
 	scheduledTransactions.inSort(strans);
 	if(strans->transaction()) {
@@ -597,40 +683,27 @@ void Budget::removeAccount(Account *account, bool keep) {
 				security = securities.next();
 			}
 		}
-		Expense *expense = expenses.first();
-		while(expense) {
-			if(expense->fromAccount() == account || expense->toAccount() == account) {
-				transactions.removeRef(expense);
-				expenses.removeRef(expense);
-				expense = expenses.current();
-			} else {
-				expense = expenses.next();
+		SplitTransaction *split = splitTransactions.first();
+		while(split) {
+			if(split->account() == account) {
+				removeSplitTransaction(split);
+				split = splitTransactions.current();
 			}
+			split = splitTransactions.next();
 		}
-		Income *income = incomes.first();
-		while(income) {
-			if(income->fromAccount() == account || income->toAccount() == account) {
-				transactions.removeRef(income);
-				incomes.removeRef(income);
-				income = incomes.current();
+		Transaction *trans = transactions.first();
+		while(trans) {
+			if(trans->fromAccount() == account || trans->toAccount() == account) {
+				removeTransaction(trans);
+				trans = transactions.current();
 			} else {
-				income = incomes.next();
-			}
-		}
-		Transfer *transfer = transfers.first();
-		while(transfer) {
-			if(transfer->fromAccount() == account || transfer->toAccount() == account) {
-				transactions.removeRef(transfer);
-				transfers.removeRef(transfer);
-				transfer = transfers.current();
-			} else {
-				transfer = transfers.next();
+				trans = transactions.next();
 			}
 		}
 		ScheduledTransaction *strans = scheduledTransactions.first();
 		while(strans) {
 			if(strans->transaction()->fromAccount() == account || strans->transaction()->toAccount() == account) {
-				scheduledTransactions.removeRef(strans);
+				removeScheduledTransaction(strans);
 				strans = scheduledTransactions.current();
 			} else {
 				strans = scheduledTransactions.next();
@@ -665,6 +738,11 @@ bool Budget::accountHasTransactions(Account *account) {
 		if(security->account() == account) return true;
 		security = securities.next();
 	}
+	SplitTransaction *split = splitTransactions.first();
+	while(split) {
+		if(split->account() == account) return true;
+		split = splitTransactions.next();
+	}
 	Transaction *trans = transactions.first();
 	while(trans) {
 		if(trans->fromAccount() == account || trans->toAccount() == account) return true;
@@ -685,6 +763,11 @@ void Budget::moveTransactions(Account *account, Account *new_account) {
 			security = securities.next();
 		}
 	}
+	SplitTransaction *split = splitTransactions.first();
+	while(split) {
+		if(split->account() == account) split->setAccount((AssetsAccount*) new_account);
+		split = splitTransactions.next();
+	}
 	Transaction *trans = transactions.first();
 	while(trans) {
 		if(trans->fromAccount() == account) trans->setFromAccount(new_account);
@@ -751,6 +834,11 @@ void Budget::scheduledTransactionDateModified(ScheduledTransaction *strans) {
 	if(scheduledTransactions.removeRef(strans)) scheduledTransactions.inSort(strans);
 	scheduledTransactions.setAutoDelete(true);
 }
+void Budget::splitTransactionDateModified(SplitTransaction *split, const QDate&) {
+	splitTransactions.setAutoDelete(false);
+	if(splitTransactions.removeRef(split)) splitTransactions.inSort(split);
+	splitTransactions.setAutoDelete(true);
+}
 void Budget::accountNameModified(Account *account) {
 	if(accounts.removeRef(account)) accounts.inSort(account);
 	switch(account->type()) {
@@ -826,6 +914,14 @@ void Budget::securityNameModified(Security *security) {
 	}
 	securities.setAutoDelete(true);
 }
+Security *Budget::findSecurity(QString name) {
+	Security *sec = securities.first();
+	while(sec) {
+		if(sec->name() == name) return sec;
+		sec = securities.next();
+	}
+	return NULL;
+}
 void Budget::addSecurityTrade(SecurityTrade *ts) {
 	securityTrades.inSort(ts);
 	ts->from_security->tradedShares.inSort(ts);
@@ -859,3 +955,35 @@ void Budget::securityTradeDateModified(SecurityTrade *ts, const QDate &olddate)
 	ts->from_security->setQuotation(ts->date, ts->value / ts->from_shares, true);
 	ts->to_security->setQuotation(ts->date, ts->value / ts->to_shares, true);
 }
+Account *Budget::findAccount(QString name) {
+	Account *account = accounts.first();
+	while(account) {
+		if(account->name() == name) return account;
+		account = accounts.next();
+	}
+	return NULL;
+}
+AssetsAccount *Budget::findAssetsAccount(QString name) {
+	AssetsAccount *account = assetsAccounts.first();
+	while(account) {
+		if(account->name() == name) return account;
+		account = assetsAccounts.next();
+	}
+	return NULL;
+}
+IncomesAccount *Budget::findIncomesAccount(QString name) {
+	IncomesAccount *account = incomesAccounts.first();
+	while(account) {
+		if(account->name() == name) return account;
+		account = incomesAccounts.next();
+	}
+	return NULL;
+}
+ExpensesAccount *Budget::findExpensesAccount(QString name) {
+	ExpensesAccount *account = expensesAccounts.first();
+	while(account) {
+		if(account->name() == name) return account;
+		account = expensesAccounts.next();
+	}
+	return NULL;
+}
