@@ -42,6 +42,19 @@ template<class type> class TransactionList : public QPtrList<type> {
 		}
 };
 
+template<class type> class SplitTransactionList : public QPtrList<type> {
+	public:
+		SplitTransactionList() : QPtrList<type>() {};
+	protected:
+		int compareItems(QPtrCollection::Item item1, QPtrCollection::Item item2) {
+			SplitTransaction *t1 = (SplitTransaction*) item1;
+			SplitTransaction *t2 = (SplitTransaction*) item2;
+			if(t1->date() > t2->date()) return 1;
+			if(t1->date() < t2->date()) return -1;
+			return 0;
+		}
+};
+
 template<class type> class ScheduledTransactionList : public QPtrList<type> {
 	public:
 		ScheduledTransactionList() : QPtrList<type>() {};
@@ -103,6 +116,9 @@ class Budget {
 
 		void addScheduledTransaction(ScheduledTransaction*);
 		void removeScheduledTransaction(ScheduledTransaction*, bool keep = false);
+
+		void addSplitTransaction(SplitTransaction*);
+		void removeSplitTransaction(SplitTransaction*, bool keep = false);
 		
 		void addAccount(Account*);
 		void removeAccount(Account*, bool keep = false);
@@ -112,11 +128,13 @@ class Budget {
 
 		void addSecurity(Security*);
 		void removeSecurity(Security*, bool keep = false);
+		Security *findSecurity(QString name);
 
 		bool securityHasTransactions(Security*);
 
 		void scheduledTransactionDateModified(ScheduledTransaction*);
 		void transactionDateModified(Transaction*, const QDate &olddate);
+		void splitTransactionDateModified(SplitTransaction*, const QDate &olddate);
 		void accountNameModified(Account*);
 		void securityNameModified(Security*);
 		void securityTradeDateModified(SecurityTrade*, const QDate &olddate);
@@ -124,6 +142,11 @@ class Budget {
 		void addSecurityTrade(SecurityTrade*);
 		void removeSecurityTrade(SecurityTrade*, bool keep = false);
 
+		Account *findAccount(QString name);
+		AssetsAccount *findAssetsAccount(QString name);
+		IncomesAccount *findIncomesAccount(QString name);
+		ExpensesAccount *findExpensesAccount(QString name);
+
 		AccountList<IncomesAccount> incomesAccounts;
 		AccountList<ExpensesAccount> expensesAccounts;
 		AccountList<AssetsAccount> assetsAccounts;
@@ -135,6 +158,7 @@ class Budget {
 		TransactionList<Transaction> transactions;
 		TransactionList<SecurityTransaction> securityTransactions;
 		ScheduledTransactionList<ScheduledTransaction> scheduledTransactions;
+		SplitTransactionList<SplitTransaction> splitTransactions;
 		SecurityList<Security> securities;
 		SecurityTradeList<SecurityTrade> securityTrades;
 
