@@ -72,6 +72,8 @@ extern QDate addYears(const QDate &date, int nyears);
 
 CategoriesComparisonChart::CategoriesComparisonChart(Budget *budg, QWidget *parent) : QWidget(parent), budget(budg) {
 
+	setWFlags(getWFlags() | Qt::WDestructiveClose);
+
 	const KCalendarSystem *calSys = KGlobal::locale()->calendar();
 
 	QDate first_date;
@@ -315,14 +317,14 @@ void CategoriesComparisonChart::save() {
 	dialog->setMimeFilter(filter, "image/png");
 	dialog->setOperationMode(KFileDialog::Saving);
 	dialog->setMode(KFile::File);
-	if(dialog->exec() != QDialog::Accepted) {delete dialog; return;}
+	if(dialog->exec() != QDialog::Accepted) {dialog->deleteLater(); return;}
 	KURL url = dialog->selectedURL();
 	QString current_filter = dialog->currentFilter();
-	delete dialog;
+	dialog->deleteLater();
 	if(url.isEmpty() && url.isValid()) return;
 	if(url.isLocalFile()) {
 		if(QFile::exists(url.path())) {
-			if(KMessageBox::warningYesNo(this, i18n("That file already exists. Would you like to overwrite the old copy?")) != KMessageBox::Yes) return;
+			if(KMessageBox::warningYesNo(this, i18n("The selected file already exists. Would you like to overwrite the old copy?")) != KMessageBox::Yes) return;
 		}
 		QFileInfo info(url.path());
 		if(info.isDir()) {
@@ -662,10 +664,14 @@ void CategoriesComparisonChart::updateDisplay() {
 	QFont legend_font = font();
 	QFontMetrics fm(legend_font);
 	int fh = fm.height();
+
+	int diameter = 430;
+	int margin = 35;
+	int legend_x = diameter + margin * 2;
 	Account *account = NULL;
 	QMap<QString, double>::iterator it_desc = desc_values.begin();
 	QMap<QString, double>::iterator it_desc_end = desc_values.end();
-	int n;
+	int n = 0;
 	if(current_account) {
 		n = desc_values.count();
 	} else if(type == ACCOUNT_TYPE_ASSETS) {
@@ -679,38 +685,53 @@ void CategoriesComparisonChart::updateDisplay() {
 		account = budget->incomesAccounts.first();
 		n = budget->incomesAccounts.count();
 	}
-	double current_value = 0.0;
-	int prev_end = 0;
 	int index = 0;
 	int text_width = 0;
+	double value_bak = value;
 	while((current_account && it_desc != it_desc_end) || account) {
-		if(current_account) current_value += it_desc.data();
-		else current_value += values[account];
-		int next_end = (int) lround((current_value * 360 * 16) / value);
-		int length = (next_end - prev_end);
-		QCanvasEllipse *ellipse = new QCanvasEllipse(400, 400, prev_end, length, canvas);
-		prev_end = next_end;
-		QCanvasText *legend_text = NULL;
+
 		bool b_empty = false;
+		QString legend_string;
+		double legend_value = 0.0;
 		if(current_account) {
-			if(it_desc.key().isEmpty()) {legend_text = new QCanvasText(i18n("No description"), legend_font, canvas); b_empty = true;}
-			else legend_text = new QCanvasText(it_desc.key(), legend_font, canvas);
+			if(it_desc.key().isEmpty()) {
+				legend_string = i18n("No description");
+			} else {
+				legend_string = it_desc.key();
+			}
+			legend_value = it_desc.data();
 		} else {
-			legend_text = new QCanvasText(account->name(), legend_font, canvas);
+			legend_string = account->name();
+			legend_value = values[account];
 		}
+		legend_value = (legend_value * 100) / value_bak;
+		int deci = 0;
+		if(legend_value < 10.0 && legend_value > -10.0) {
+			legend_value = round(legend_value * 10.0) / 10.0;
+			deci = 1;
+		} else {
+			legend_value = round(legend_value);
+		}
+		QCanvasText *legend_text = new QCanvasText(QString("%1 (%2\%)").arg(legend_string).arg(KGlobal::locale()->formatNumber(legend_value, deci)), legend_font, canvas);
+		
 		int index_bak = index;
 		if(b_empty) index = n - 1;
-		ellipse->setBrush(getBrush(index));
-		ellipse->move(250, 250);
-		ellipse->show();
-		QCanvasRectangle *legend_box = new QCanvasRectangle(510, 60 + (fh + 5) * index, fh, fh, canvas);
-		legend_box->setPen(QPen(Qt::black));
-		legend_box->setBrush(getBrush(index));
-		legend_box->show();
 		if(legend_text->boundingRect().width() > text_width) text_width = legend_text->boundingRect().width();
 		legend_text->setColor(Qt::black);
-		legend_text->move(510 + fh + 5, 60 + (fh + 5) * index);
+		legend_text->move(legend_x + 10 + fh + 5, margin + 10 + (fh + 5) * index);
 		legend_text->show();
+		
+		if(current_account) {
+			if(it_desc.data() < 0.0) {
+				value -= it_desc.data();
+				*it_desc = 0.0;
+			}
+		} else {
+			if(values[account] < 0.0) {
+				value -= values[account];
+				values[account] = 0.0;
+			}
+		}
 		if(current_account) {
 			++it_desc;
 		} else if(type == ACCOUNT_TYPE_ASSETS) {
@@ -722,14 +743,57 @@ void CategoriesComparisonChart::updateDisplay() {
 		else index++;
 	}
 
-	QCanvasRectangle *legend_outline = new QCanvasRectangle(500, 50, 10 + fh + 5 + text_width + 10, 10 + ((fh + 5) * n) + 5, canvas);
+	if(current_account) {
+	} else if(type == ACCOUNT_TYPE_ASSETS) {
+		account = budget->assetsAccounts.first();
+		if(account == budget->balancingAccount) account = budget->assetsAccounts.next();
+	} else if(type == ACCOUNT_TYPE_EXPENSES) {
+		account = budget->expensesAccounts.first();
+	} else {
+		account = budget->incomesAccounts.first();
+	}
+	it_desc = desc_values.begin();
+	it_desc_end = desc_values.end();
+	index = 0;
+	double current_value = 0.0;
+	int prev_end = 0;
+	while((current_account && it_desc != it_desc_end) || account) {
+		if(current_account) current_value += it_desc.data();
+		else current_value += values[account];
+		int next_end = (int) lround((current_value * 360 * 16) / value);
+		int length = (next_end - prev_end);
+		QCanvasEllipse *ellipse = new QCanvasEllipse(diameter, diameter, prev_end, length, canvas);
+		prev_end = next_end;
+		bool b_empty = false;
+		if(current_account && it_desc.key().isEmpty()) {b_empty = true;}
+		int index_bak = index;
+		if(b_empty) index = n - 1;
+		ellipse->setBrush(getBrush(index));
+		ellipse->move(diameter / 2 + margin, diameter / 2 + margin);
+		ellipse->show();
+		QCanvasRectangle *legend_box = new QCanvasRectangle(legend_x + 10, margin + 10 + (fh + 5) * index, fh, fh, canvas);
+		legend_box->setPen(QPen(Qt::black));
+		legend_box->setBrush(getBrush(index));
+		legend_box->show();
+		if(current_account) {
+			++it_desc;
+		} else if(type == ACCOUNT_TYPE_ASSETS) {
+			account = budget->assetsAccounts.next();
+			if(account == budget->balancingAccount) account = budget->assetsAccounts.next();
+		} else if(type == ACCOUNT_TYPE_EXPENSES) account = budget->expensesAccounts.next();
+		else  account = budget->incomesAccounts.next();
+		if(b_empty) index = index_bak;
+		else index++;
+	}
+
+	QCanvasRectangle *legend_outline = new QCanvasRectangle(legend_x, margin, 10 + fh + 5 + text_width + 10, 10 + ((fh + 5) * n) + 5, canvas);
 	legend_outline->setPen(QPen(Qt::black));
 	legend_outline->show();
 
 	double ratio = (double) view->visibleWidth() / (double) view->visibleHeight();
-	min_width = (int) ceil(legend_outline->x() + legend_outline->width() + 50);
-	min_height = (int) ceil(legend_outline->y() + legend_outline->height() + 50);
-	if(min_height < 500) min_height = 500;
+	min_width = (int) ceil(legend_outline->x() + legend_outline->width() + margin);
+	min_height = (int) ceil(legend_outline->y() + legend_outline->height() + margin);
+	if(min_height < diameter + margin * 2) min_height = diameter + margin * 2;
 	int height, width;
 	if(floor(min_height * ratio) < min_width) {
 		height = (int) floor(min_width / ratio);
