@@ -70,6 +70,8 @@ extern QDate addYears(const QDate &date, int nyears);
 
 CategoriesComparisonReport::CategoriesComparisonReport(Budget *budg, QWidget *parent, bool extra_parameters) : QWidget(parent), budget(budg), b_extra(extra_parameters) {
 
+	setWFlags(getWFlags() | Qt::WDestructiveClose);
+
 	const KCalendarSystem *calSys = KGlobal::locale()->calendar();
 
 	QDate first_date;
@@ -450,13 +452,13 @@ void CategoriesComparisonReport::save() {
 	dialog->setMimeFilter(filter, "text/html");
 	dialog->setOperationMode(KFileDialog::Saving);
 	dialog->setMode(KFile::File);
-	if(dialog->exec() != QDialog::Accepted) {delete dialog; return;}
+	if(dialog->exec() != QDialog::Accepted) {dialog->deleteLater(); return;}
 	KURL url = dialog->selectedURL();
-	delete dialog;
+	dialog->deleteLater();
 	if(url.isEmpty() && url.isValid()) return;
 	if(url.isLocalFile()) {
 		if(QFile::exists(url.path())) {
-			if(KMessageBox::warningYesNo(this, i18n("That file already exists. Would you like to overwrite the old copy?")) != KMessageBox::Yes) return;
+			if(KMessageBox::warningYesNo(this, i18n("The selected file already exists. Would you like to overwrite the old copy?")) != KMessageBox::Yes) return;
 		}
 		QFileInfo info(url.path());
 		if(info.isDir()) {
