@@ -34,6 +34,7 @@
 class Budget;
 class Transaction;
 class ScheduledTransaction;
+class SplitTransaction;
 class Account;
 class AssetsAccount;
 class IncomesAccount;
@@ -80,7 +81,7 @@ class Eqonomize : public KMainWindow {
 		virtual ~Eqonomize();
 		
 		bool saveURL(const KURL& url);
-		bool askSave();
+		bool askSave(bool before_exit = false);
 		void createDefaultBudget();
 		void readFileDependentOptions();
 
@@ -102,21 +103,27 @@ class Eqonomize : public KMainWindow {
 		void updateMonthlyBudget(Account *account);
 		void updateTotalMonthlyExpensesBudget();
 		void updateTotalMonthlyIncomesBudget();
-		void editAccount(Account*);
+		bool editAccount(Account*);
+		bool editAccount(Account*, QWidget *parent);
 		void balanceAccount(Account*);
 		bool checkSchedule(bool update_display);
 		void updateScheduledTransactions();
 		void appendScheduledTransaction(ScheduledTransaction *strans);
-		void editScheduledTransaction(ScheduledTransaction *strans);
+		bool editScheduledTransaction(ScheduledTransaction *strans);
 		bool editScheduledTransaction(ScheduledTransaction *strans, QWidget *parent);
-		void editOccurrence(ScheduledTransaction *strans, const QDate &date);
+		bool editOccurrence(ScheduledTransaction *strans, const QDate &date);
 		bool editOccurrence(ScheduledTransaction *strans, const QDate &date, QWidget *parent);
 		bool editTransaction(Transaction *trans, QWidget *parent);
-		void editTransaction(Transaction *trans);
-		void removeScheduledTransaction(ScheduledTransaction *strans);
-		void removeOccurrence(ScheduledTransaction *strans, const QDate &date);
-		void newScheduledTransaction(int transaction_type, Security *security = NULL);
-		void newScheduledTransaction(int transaction_type, Security *security, QWidget *parent);
+		bool editTransaction(Transaction *trans);
+		bool removeScheduledTransaction(ScheduledTransaction *strans);
+		bool removeOccurrence(ScheduledTransaction *strans, const QDate &date);
+		bool newScheduledTransaction(int transaction_type, Security *security = NULL);
+		bool newScheduledTransaction(int transaction_type, Security *security, QWidget *parent, Account *account = NULL);
+		bool newSplitTransaction(QWidget *parent, AssetsAccount *account = NULL);
+		bool editSplitTransaction(SplitTransaction *split);
+		bool editSplitTransaction(SplitTransaction *split, QWidget *parent);
+		bool splitUpTransaction(SplitTransaction *split);
+		bool removeSplitTransaction(SplitTransaction *split);
 		bool saveView(QTextStream &file, int fileformat);
 		bool exportScheduleList(QTextStream &outf, int fileformat);
 		bool exportAccountsList(QTextStream &outf, int fileformat);
@@ -135,14 +142,20 @@ class Eqonomize : public KMainWindow {
 		void showIncomes();
 		void showTransfers();
 		void updateSecuritiesStatistics();
+		bool crashRecovery(KURL url);
+		bool newRefundRepayment(Transaction *trans);
 
 		KAction *ActionAddAccount, *ActionNewAssetsAccount, *ActionNewIncomesAccount, *ActionNewExpensesAccount, *ActionEditAccount, *ActionDeleteAccount, *ActionBalanceAccount;
-		KAction *ActionShowAccountExpenses, *ActionShowAccountIncomes;
-		KAction *ActionNewExpense, *ActionNewIncome, *ActionNewTransfer, *ActionEditTransaction, *ActionEditScheduledTransaction, *ActionDeleteTransaction, *ActionDeleteScheduledTransaction;
+		KAction *ActionShowAccountTransactions;
+		KAction *ActionNewExpense, *ActionNewIncome, *ActionNewTransfer, *ActionNewSplitTransaction;
+		KAction *ActionEditTransaction, *ActionEditScheduledTransaction, *ActionEditSplitTransaction;
+		KAction *ActionJoinTransactions, *ActionSplitUpTransaction;
+		KAction *ActionDeleteTransaction, *ActionDeleteScheduledTransaction, *ActionDeleteSplitTransaction;
 		KAction *ActionNewSecurity, *ActionEditSecurity, *ActionBuyShares, *ActionSellShares, *ActionNewDividend, *ActionNewReinvestedDividend, *ActionNewSecurityTrade, *ActionSetQuotation, *ActionEditQuotations, *ActionEditSecurityTransactions, *ActionDeleteSecurity;
-		KAction *ActionSave, *ActionSaveView, *ActionPrintView;
+		KAction *ActionNewRefund, *ActionNewRepayment, *ActionNewRefundRepayment;
+		KAction *ActionSave, *ActionFileReload, *ActionSaveView, *ActionPrintView;
 		KAction *ActionOverTimeReport, *ActionCategoriesComparisonReport, *ActionOverTimeChart, *ActionCategoriesComparisonChart;
-		KAction *ActionImportCSV;
+		KAction *ActionImportCSV, *ActionImportQIF, *ActionExportQIF;
 		KToggleAction *ActionExtraProperties;
 		KRecentFilesAction *ActionOpenRecent;
 
@@ -161,12 +174,14 @@ class Eqonomize : public KMainWindow {
 		KConfig *config;
 		KURL current_url;
 		double period_months, from_to_months;
-		bool modified;
+		bool modified, modified_auto_save, auto_save_timeout;
 		QDate from_date, to_date, frommonth_begin, prevmonth_begin;
 		QDate securities_from_date, securities_to_date;
+		QDate prev_cur_date;
 		int current_page;
 		bool partial_budget;
 		bool b_extra;
+		QString cr_tmp_file;
 
 		KToolBar *scheduleButtons, *securitiesButtons;
 		KJanusWidget *tabs;
@@ -213,9 +228,15 @@ class Eqonomize : public KMainWindow {
 
 	public slots:
 
+		void saveCrashRecovery();
+		void autoSave();
+		void onAutoSaveTimeout();
+
 		void useExtraProperties(bool);
 
 		void importCSV();
+		void importQIF();
+		void exportQIF();
 
 		void reloadBudget();
 
@@ -246,6 +267,7 @@ class Eqonomize : public KMainWindow {
 		void popupSecuritiesMenu(KListView*, QListViewItem*, const QPoint&);
 		void updateSecurities();
 		
+		void newSplitTransaction();
 		void newScheduledExpense();
 		void newScheduledIncome();
 		void newScheduledTransfer();
@@ -259,14 +281,21 @@ class Eqonomize : public KMainWindow {
 
 		void editSelectedScheduledTransaction();
 		void editSelectedTransaction();
+		void editSelectedSplitTransaction();
 		void deleteSelectedScheduledTransaction();
 		void deleteSelectedTransaction();
+		void deleteSelectedSplitTransaction();
+		void joinSelectedTransactions();
+		void splitUpSelectedTransaction();
+
+		void newRefund();
+		void newRepayment();
+		void newRefundRepayment();
 
 		void checkForPageChange();
 		void onPageChange(QWidget*);
 
-		void showAccountExpenses(bool = false);
-		void showAccountIncomes(bool = false);
+		void showAccountTransactions(bool = false);
 
 		void updateTransactionActions();
 		
@@ -274,12 +303,15 @@ class Eqonomize : public KMainWindow {
 		void fileNew();
 		void fileOpen();
 		void fileOpenRecent(const KURL&);
+		void fileReload();
 		void fileSave();
 		void fileSaveAs();
 		void optionsPreferences();
 
 		void checkSchedule();
 
+		void checkDate();
+
 		void popupAccountsMenu(KListView*, QListViewItem*, const QPoint&);
 
 		void addAccount();
@@ -328,6 +360,9 @@ class Eqonomize : public KMainWindow {
 		void scheduledTransactionModified(ScheduledTransaction*, ScheduledTransaction*);
 		void scheduledTransactionRemoved(ScheduledTransaction*);
 
+		void splitTransactionAdded(SplitTransaction*);
+		void splitTransactionRemoved(SplitTransaction*);
+
 		void filterAccounts();
 
 	signals:
@@ -348,7 +383,7 @@ class ConfirmScheduleDialog : public KDialogBase {
 		KListView *transactionsView;
 		Budget *budget;
 		bool b_extra;
-		QPushButton *editButton, *removeButton;
+		QPushButton *editButton, *removeButton, *postponeButton;
 		ConfirmScheduleListViewItem *current_item;
 		
 	public:
@@ -363,6 +398,7 @@ class ConfirmScheduleDialog : public KDialogBase {
 		void transactionSelectionChanged();
 		void remove();
 		void edit();
+		void postpone();
 		void updateTransactions();
 		
 };
@@ -379,6 +415,7 @@ class EditAssetsAccountDialog : public KDialogBase {
 		KComboBox *typeCombo;
 		QCheckBox *budgetButton;
 		Budget *budget;
+		Account *current_account;
 		
 	public:
 		
@@ -391,6 +428,7 @@ class EditAssetsAccountDialog : public KDialogBase {
 	protected slots:
 
 		void typeActivated(int);
+		void slotOk();
 		
 };
 
@@ -405,6 +443,7 @@ class EditExpensesAccountDialog : public KDialogBase {
 		KTextEdit *descriptionEdit;
 		QCheckBox *budgetButton;
 		Budget *budget;
+		Account *current_account;
 		
 	public:
 		
@@ -416,6 +455,7 @@ class EditExpensesAccountDialog : public KDialogBase {
 	protected slots:
 
 		void budgetEnabled(bool);
+		void slotOk();
 		
 };
 
@@ -430,6 +470,7 @@ class EditIncomesAccountDialog : public KDialogBase {
 		KTextEdit *descriptionEdit;
 		QCheckBox *budgetButton;
 		Budget *budget;
+		Account *current_account;
 		
 	public:
 		
@@ -441,6 +482,7 @@ class EditIncomesAccountDialog : public KDialogBase {
 	protected slots:
 
 		void budgetEnabled(bool);
+		void slotOk();
 		
 };
 
@@ -456,7 +498,7 @@ class EditSecurityDialog : public KDialogBase {
 		KDateEdit *quotationDateEdit;
 		KComboBox *typeCombo, *accountCombo;
 		QSpinBox *decimalsEdit;
-		QLabel *quotationLabel;
+		QLabel *quotationLabel, *quotationDateLabel;
 		Budget *budget;
 		QValueVector<AssetsAccount*> accounts;
 
@@ -502,6 +544,31 @@ class EditQuotationsDialog : public KDialogBase {
 
 };
 
+class RefundDialog : public KDialogBase {
+
+	Q_OBJECT
+
+	protected:
+
+		Transaction *transaction;
+		KDoubleSpinBox *valueEdit, *quantityEdit;
+		KDateEdit *dateEdit;
+		KComboBox *accountCombo;
+		KLineEdit *commentsEdit;
+
+	public:
+
+		RefundDialog(Transaction *trans, QWidget *parent);
+
+		Transaction *createRefund();
+		bool validValues();
+
+	protected slots:
+		
+		void slotOk();
+
+};
+
 class EditReinvestedDividendDialog : public KDialogBase {
 
 	Q_OBJECT
