@@ -75,6 +75,8 @@ extern QDate addMonths(const QDate &date, int nmonths);
 
 OverTimeChart::OverTimeChart(Budget *budg, QWidget *parent, bool extra_parameters) : QWidget(parent), budget(budg), b_extra(extra_parameters) {
 
+	setWFlags(getWFlags() | Qt::WDestructiveClose);
+
 	const KCalendarSystem *calSys = KGlobal::locale()->calendar();
 	
 	QVBoxLayout *layout = new QVBoxLayout(this, 6, 6);
@@ -446,7 +448,7 @@ void OverTimeChart::startYearChanged(const QDate &date) {
 			updateDisplay();
 			return;
 		} else {
-			KMessageBox::error(this, i18n("Future dates is not allowed."));
+			KMessageBox::error(this, i18n("Future dates are not allowed."));
 			error = true;
 		}
 	}
@@ -467,7 +469,7 @@ void OverTimeChart::startMonthChanged(const QDate &date) {
 		error = true;
 	}
 	if(!error && date > QDate::currentDate()) {
-		KMessageBox::error(this, i18n("Future dates is not allowed."));
+		KMessageBox::error(this, i18n("Future dates are not allowed."));
 		error = true;
 	}
 	if(error) {
@@ -508,14 +510,14 @@ void OverTimeChart::save() {
 	dialog->setMimeFilter(filter, "image/png");
 	dialog->setOperationMode(KFileDialog::Saving);
 	dialog->setMode(KFile::File);
-	if(dialog->exec() != QDialog::Accepted) {delete dialog; return;}
+	if(dialog->exec() != QDialog::Accepted) {dialog->deleteLater(); return;}
 	KURL url = dialog->selectedURL();
 	QString current_filter = dialog->currentFilter();
-	delete dialog;
+	dialog->deleteLater();
 	if(url.isEmpty() && url.isValid()) return;
 	if(url.isLocalFile()) {
 		if(QFile::exists(url.path())) {
-			if(KMessageBox::warningYesNo(this, i18n("That file already exists. Would you like to overwrite the old copy?")) != KMessageBox::Yes) return;
+			if(KMessageBox::warningYesNo(this, i18n("The selected file already exists. Would you like to overwrite the old copy?")) != KMessageBox::Yes) return;
 		}
 		QFileInfo info(url.path());
 		if(info.isDir()) {
@@ -1339,8 +1341,8 @@ void OverTimeChart::updateDisplay() {
 			maxvalue /= 2.0;
 			maxvalue = ceil(maxvalue) * 2.0;
 		} else {
-			maxvalue /= exp10(maxvalue_exp);
-			maxvalue = ceil(maxvalue) * exp10(maxvalue_exp);
+			maxvalue /= pow(10, maxvalue_exp);
+			maxvalue = ceil(maxvalue) * pow(10, maxvalue_exp);
 		}
 		if(minvalue < 0.0) minvalue = -maxvalue;
 	}
