@@ -70,6 +70,8 @@ struct month_info {
 };
 
 OverTimeReport::OverTimeReport(Budget *budg, QWidget *parent) : QWidget(parent), budget(budg) {
+
+	setWFlags(getWFlags() | Qt::WDestructiveClose);
 	
 	QVBoxLayout *layout = new QVBoxLayout(this, 6, 6);
 	
@@ -267,13 +269,13 @@ void OverTimeReport::save() {
 	dialog->setMimeFilter(filter, "text/html");
 	dialog->setOperationMode(KFileDialog::Saving);
 	dialog->setMode(KFile::File);
-	if(dialog->exec() != QDialog::Accepted) {delete dialog; return;}
+	if(dialog->exec() != QDialog::Accepted) {dialog->deleteLater(); return;}
 	KURL url = dialog->selectedURL();
-	delete dialog;
+	dialog->deleteLater();
 	if(url.isEmpty() && url.isValid()) return;
 	if(url.isLocalFile()) {
 		if(QFile::exists(url.path())) {
-			if(KMessageBox::warningYesNo(this, i18n("That file already exists. Would you like to overwrite the old copy?")) != KMessageBox::Yes) return;
+			if(KMessageBox::warningYesNo(this, i18n("The selected file already exists. Would you like to overwrite the old copy?")) != KMessageBox::Yes) return;
 		}
 		QFileInfo info(url.path());
 		if(info.isDir()) {
