@@ -53,8 +53,10 @@ Security::Security(Budget *parent_budget, QDomElement *e, bool *valid) : o_budge
 		st_type = SECURITY_TYPE_BOND;
 	} else if(type == "stock") {
 		st_type = SECURITY_TYPE_STOCK;
-	} else {
+	} else if(type == "mutual fund") {
 		st_type = SECURITY_TYPE_MUTUAL_FUND;
+	} else {
+		st_type = SECURITY_TYPE_OTHER;
 	}
 	d_initial_shares = e->attribute("initialshares").toDouble();
 	i_decimals = e->attribute("decimals", "4").toInt();
@@ -156,6 +158,7 @@ void Security::save(QDomElement *e) const {
 		case SECURITY_TYPE_BOND: {e->setAttribute("type", "bond"); break;}
 		case SECURITY_TYPE_STOCK: {e->setAttribute("type", "stock"); break;}
 		case SECURITY_TYPE_MUTUAL_FUND: {e->setAttribute("type", "mutual fund"); break;}
+		case SECURITY_TYPE_OTHER: {e->setAttribute("type", "other"); break;}
 	}
 	e->setAttribute("decimals", i_decimals);
 	e->setAttribute("initialshares", QString::number(d_initial_shares, 'f', i_decimals));
