@@ -32,8 +32,11 @@
 #include <klocale.h>
 #include <kglobal.h>
 
-Transaction::Transaction(Budget *parent_budget, double initial_value, QDate initial_date, Account *from, Account *to, QString initial_description, QString initial_comment) : o_budget(parent_budget), d_value(initial_value), d_date(initial_date), o_from(from), o_to(to), s_description(initial_description.stripWhiteSpace()), s_comment(initial_comment), d_quantity(1.0) {}
+Transaction::Transaction(Budget *parent_budget, double initial_value, QDate initial_date, Account *from, Account *to, QString initial_description, QString initial_comment) : o_budget(parent_budget), d_value(initial_value), d_date(initial_date), o_from(from), o_to(to), s_description(initial_description.stripWhiteSpace()), s_comment(initial_comment), d_quantity(1.0), o_split(NULL) {
+	if(s_description.isNull()) s_description = emptystr;
+}
 Transaction::Transaction(Budget *parent_budget, QDomElement *e, bool *valid) : o_budget(parent_budget) {
+	o_split = NULL;
 	o_from = NULL; o_to = NULL;
 	d_date = QDate::fromString(e->attribute("date"), Qt::ISODate);
 	s_description = e->attribute("description", emptystr).stripWhiteSpace();
@@ -41,10 +44,25 @@ Transaction::Transaction(Budget *parent_budget, QDomElement *e, bool *valid) : o
 	d_quantity = e->attribute("quantity", "1.0").toDouble();
 	if(valid) *valid = d_date.isValid();
 }
-Transaction::Transaction() : o_budget(NULL), o_from(NULL), o_to(NULL), d_quantity(1.0) {}
-Transaction::Transaction(const Transaction *transaction) : o_budget(transaction->budget()), d_value(transaction->value()), d_date(transaction->date()), o_from(transaction->fromAccount()), o_to(transaction->toAccount()), s_description(transaction->description()), s_comment(transaction->comment()), d_quantity(transaction->quantity()) {}
+Transaction::Transaction() : o_budget(NULL), d_value(0.0), o_from(NULL), o_to(NULL), s_description(emptystr), d_quantity(1.0), o_split(NULL) {}
+Transaction::Transaction(const Transaction *transaction) : o_budget(transaction->budget()), d_value(transaction->value()), d_date(transaction->date()), o_from(transaction->fromAccount()), o_to(transaction->toAccount()), s_description(transaction->description()), s_comment(transaction->comment()), d_quantity(transaction->quantity()), o_split(NULL) {}
 Transaction::~Transaction() {}
-		
+
+bool Transaction::equals(const Transaction *transaction) const {
+	if(type() != transaction->type()) return false;
+	if(fromAccount() != transaction->fromAccount()) return false;
+	if(toAccount() != transaction->toAccount()) return false;
+	if(value() != transaction->value()) return false;
+	if(date() != transaction->date()) return false;
+	if(quantity() != transaction->quantity()) return false;
+	if(description() != transaction->description()) return false;	
+	if(comment() != transaction->comment() && comment().isEmpty() != transaction->comment().isEmpty()) return false;
+	if(budget() != transaction->budget()) return false;
+	return true;
+}
+
+SplitTransaction *Transaction::parentSplit() const {return o_split;}
+void Transaction::setParentSplit(SplitTransaction *parent) {o_split = parent;}
 double Transaction::value() const {return d_value;}
 void Transaction::setValue(double new_value) {d_value = new_value;}
 double Transaction::quantity() const {return d_quantity;}
@@ -63,27 +81,37 @@ Budget *Transaction::budget() const {return o_budget;}
 void Transaction::save(QDomElement *e) const {
 	e->setAttribute("date", d_date.toString(Qt::ISODate));
 	if(!s_comment.isEmpty())  e->setAttribute("comment", s_comment);
-	if(d_quantity != 1.0) e->setAttribute("quantity", d_quantity);
+	if(d_quantity != 1.0) e->setAttribute("quantity", QString::number(d_quantity, 'f', KGlobal::locale()->fracDigits()));
 }
 
-Expense::Expense(Budget *parent_budget, double initial_cost, QDate initial_date, ExpensesAccount *initial_category, AssetsAccount *initial_from, QString initial_description, QString initial_comment) : Transaction(parent_budget, initial_cost, initial_date, initial_from, initial_category, initial_description, initial_comment), s_payee(emptystr) {}
+Expense::Expense(Budget *parent_budget, double initial_cost, QDate initial_date, ExpensesAccount *initial_category, AssetsAccount *initial_from, QString initial_description, QString initial_comment) : Transaction(parent_budget, initial_cost, initial_date, initial_from, initial_category, initial_description, initial_comment), s_payee(emptystr) {
+	if(s_payee.isNull()) s_payee = emptystr;
+}
 Expense::Expense(Budget *parent_budget, QDomElement *e, bool *valid) : Transaction(parent_budget, e, valid) {
 	int id_category = e->attribute("category").toInt();
 	int id_from = e->attribute("from").toInt();
 	if(parent_budget->expensesAccounts_id.contains(id_category) && parent_budget->assetsAccounts_id.contains(id_from)) {
 		setCategory(parent_budget->expensesAccounts_id[id_category]);
 		setFrom(parent_budget->assetsAccounts_id[id_from]);
-		setCost(e->attribute("cost").toDouble());
+		if(e->hasAttribute("income")) setCost(-e->attribute("income").toDouble());
+		else setCost(e->attribute("cost").toDouble());
 		s_payee = e->attribute("payee", emptystr);
 	} else {
 		if(valid) *valid =false;
 	}
 }
-Expense::Expense() : Transaction() {}
+Expense::Expense() : Transaction(), s_payee(emptystr) {}
 Expense::Expense(const Expense *expense) : Transaction(expense), s_payee(expense->payee()) {}
 Expense::~Expense() {}
 Transaction *Expense::copy() const {return new Expense(this);}
 
+bool Expense::equals(const Transaction *transaction) const {
+	if(!Transaction::equals(transaction)) return false;
+	Expense *expense = (Expense*) transaction;
+	if(s_payee != expense->payee()) return false;
+	return true;
+}
+
 ExpensesAccount *Expense::category() const {return (ExpensesAccount*) toAccount();}
 void Expense::setCategory(ExpensesAccount *new_category) {setToAccount(new_category);}
 AssetsAccount *Expense::from() const {return (AssetsAccount*) fromAccount();}
@@ -95,21 +123,25 @@ void Expense::setPayee(QString new_payee) {s_payee = new_payee;}
 TransactionType Expense::type() const {return TRANSACTION_TYPE_EXPENSE;}
 void Expense::save(QDomElement *e) const {
 	Transaction::save(e);
-	e->setAttribute("cost", QString::number(cost(), 'f', KGlobal::locale()->fracDigits()));
+	if(cost() < 0.0) e->setAttribute("income", QString::number(-cost(), 'f', KGlobal::locale()->fracDigits()));
+	else e->setAttribute("cost", QString::number(cost(), 'f', KGlobal::locale()->fracDigits()));
 	e->setAttribute("category", category()->id());
 	e->setAttribute("from", from()->id());
 	if(!s_description.isEmpty()) e->setAttribute("description", s_description);
 	if(!s_payee.isEmpty()) e->setAttribute("payee", s_payee);	
 }
 
-Income::Income(Budget *parent_budget, double initial_income, QDate initial_date, IncomesAccount *initial_category, AssetsAccount *initial_to, QString initial_description, QString initial_comment) : Transaction(parent_budget, initial_income, initial_date, initial_category, initial_to, initial_description, initial_comment), o_security(NULL), s_payer(emptystr) {}
+Income::Income(Budget *parent_budget, double initial_income, QDate initial_date, IncomesAccount *initial_category, AssetsAccount *initial_to, QString initial_description, QString initial_comment) : Transaction(parent_budget, initial_income, initial_date, initial_category, initial_to, initial_description, initial_comment), o_security(NULL), s_payer(emptystr) {
+	if(s_payer.isNull()) s_payer = emptystr;
+}
 Income::Income(Budget *parent_budget, QDomElement *e, bool *valid) : Transaction(parent_budget, e, valid) {
 	int id_category = e->attribute("category").toInt();
 	int id_to = e->attribute("to").toInt();
 	if(parent_budget->incomesAccounts_id.contains(id_category) && parent_budget->assetsAccounts_id.contains(id_to)) {
 		setCategory(parent_budget->incomesAccounts_id[id_category]);
 		setTo(parent_budget->assetsAccounts_id[id_to]);
-		setIncome(e->attribute("income").toDouble());
+		if(e->hasAttribute("cost")) setIncome(-e->attribute("cost").toDouble());
+		else setIncome(e->attribute("income").toDouble());
 	} else {
 		if(valid) *valid =false;
 	}
@@ -126,11 +158,19 @@ Income::Income(Budget *parent_budget, QDomElement *e, bool *valid) : Transaction
 		s_payer = e->attribute("payer", emptystr);
 	}
 }
-Income::Income() : Transaction(), o_security(NULL) {}
+Income::Income() : Transaction(), o_security(NULL), s_payer(emptystr) {}
 Income::Income(const Income *income_) : Transaction(income_), o_security(income_->security()), s_payer(income_->payer()) {}
 Income::~Income() {}
 Transaction *Income::copy() const {return new Income(this);}
 
+bool Income::equals(const Transaction *transaction) const {
+	if(!Transaction::equals(transaction)) return false;
+	Income *income = (Income*) transaction;
+	if(s_payer != income->payer()) return false;
+	if(o_security != income->security()) return false;
+	return true;
+}
+
 IncomesAccount *Income::category() const {return (IncomesAccount*) fromAccount();}
 void Income::setCategory(IncomesAccount *new_category) {setFromAccount(new_category);}
 AssetsAccount *Income::to() const {return (AssetsAccount*) toAccount();}
@@ -149,7 +189,8 @@ void Income::setSecurity(Security *parent_security) {
 Security *Income::security() const {return o_security;}
 void Income::save(QDomElement *e) const {
 	Transaction::save(e);
-	e->setAttribute("income", QString::number(income(), 'f', KGlobal::locale()->fracDigits()));
+	if(income() < 0.0 && !o_security) e->setAttribute("cost", QString::number(-income(), 'f', KGlobal::locale()->fracDigits()));
+	else e->setAttribute("income", QString::number(income(), 'f', KGlobal::locale()->fracDigits()));
 	e->setAttribute("category", category()->id());
 	e->setAttribute("to", to()->id());
 	if(o_security) {
@@ -160,15 +201,21 @@ void Income::save(QDomElement *e) const {
 	}
 }
 
-Transfer::Transfer(Budget *parent_budget, double initial_amount, QDate initial_date, AssetsAccount *initial_from, AssetsAccount *initial_to, QString initial_description, QString initial_comment) : Transaction(parent_budget, initial_amount, initial_date, initial_from, initial_to, initial_description, initial_comment) {}
+Transfer::Transfer(Budget *parent_budget, double initial_amount, QDate initial_date, AssetsAccount *initial_from, AssetsAccount *initial_to, QString initial_description, QString initial_comment) : Transaction(parent_budget, initial_amount < 0.0 ? -initial_amount : initial_amount, initial_date, initial_amount < 0.0 ? initial_to : initial_from, initial_amount < 0.0 ? initial_from : initial_to, initial_description, initial_comment) {}
 Transfer::Transfer(Budget *parent_budget, QDomElement *e, bool *valid, bool internal_is_balancing) : Transaction(parent_budget, e, valid) {
 	if(internal_is_balancing) {setAmount(e->attribute("amount").toDouble()); return;}
 	int id_from = e->attribute("from").toInt();
 	int id_to = e->attribute("to").toInt();
 	if(parent_budget->assetsAccounts_id.contains(id_from) && parent_budget->assetsAccounts_id.contains(id_to)) {
-		setFrom(parent_budget->assetsAccounts_id[id_from]);
-		setTo(parent_budget->assetsAccounts_id[id_to]);
 		setAmount(e->attribute("amount").toDouble());
+		if(amount() < 0.0) {
+			setAmount(-amount());
+			setFrom(parent_budget->assetsAccounts_id[id_to]);
+			setTo(parent_budget->assetsAccounts_id[id_from]);
+		} else {
+			setFrom(parent_budget->assetsAccounts_id[id_from]);
+			setTo(parent_budget->assetsAccounts_id[id_to]);
+		}
 	} else {
 		if(valid) *valid =false;
 	}
@@ -183,7 +230,16 @@ void Transfer::setTo(AssetsAccount *new_to) {setToAccount(new_to);}
 AssetsAccount *Transfer::from() const {return (AssetsAccount*) fromAccount();}
 void Transfer::setFrom(AssetsAccount *new_from) {setFromAccount(new_from);}
 double Transfer::amount() const {return value();}
-void Transfer::setAmount(double new_amount) {setValue(new_amount);}
+void Transfer::setAmount(double new_amount) {
+	if(new_amount < 0.0) {
+		setValue(-new_amount);
+		AssetsAccount *from_bak = from();
+		setFrom(to());
+		setTo(from_bak);
+	} else {
+		setValue(new_amount);
+	}
+}
 TransactionType Transfer::type() const {return TRANSACTION_TYPE_TRANSFER;}
 void Transfer::save(QDomElement *e) const {
 	Transaction::save(e);
@@ -225,10 +281,11 @@ void Balancing::save(QDomElement *e) const {
 	if(s_description != i18n("Account balancing")) e->setAttribute("description", s_description);
 }
 
-SecurityTransaction::SecurityTransaction(Security *parent_security, double initial_value, double initial_shares, QDate initial_date, QString initial_comment) : Transaction(parent_security->budget(), initial_value, initial_date, NULL, NULL, QString::null, initial_comment), o_security(parent_security), d_shares(initial_shares) {
+SecurityTransaction::SecurityTransaction(Security *parent_security, double initial_value, double initial_shares, double initial_share_value, QDate initial_date, QString initial_comment) : Transaction(parent_security->budget(), initial_value, initial_date, NULL, NULL, QString::null, initial_comment), o_security(parent_security), d_shares(initial_shares), d_share_value(initial_share_value) {
 }
 SecurityTransaction::SecurityTransaction(Budget *parent_budget, QDomElement *e, bool *valid) : Transaction(parent_budget, e, valid) {
 	d_shares = e->attribute("shares").toDouble();
+	d_share_value = e->attribute("sharevalue", "-1.0").toDouble();
 	int id = e->attribute("security").toInt();
 	if(parent_budget->securities_id.contains(id)) {
 		o_security = parent_budget->securities_id[id];
@@ -236,24 +293,27 @@ SecurityTransaction::SecurityTransaction(Budget *parent_budget, QDomElement *e,
 		if(valid) *valid =false;
 	}
 }
-SecurityTransaction::SecurityTransaction() : Transaction(), o_security(NULL), d_shares(0.0) {}
-SecurityTransaction::SecurityTransaction(const SecurityTransaction *transaction) : Transaction(transaction), o_security(transaction->security()), d_shares(transaction->shares()) {}
+SecurityTransaction::SecurityTransaction() : Transaction(), o_security(NULL), d_shares(0.0), d_share_value(0.0) {}
+SecurityTransaction::SecurityTransaction(const SecurityTransaction *transaction) : Transaction(transaction), o_security(transaction->security()), d_shares(transaction->shares()), d_share_value(transaction->shareValue()) {}
 SecurityTransaction::~SecurityTransaction() {}
 
-double SecurityTransaction::value() const {return d_value;}
+bool SecurityTransaction::equals(const Transaction *transaction) const {
+	if(!Transaction::equals(transaction)) return false;
+	SecurityTransaction *sectrans = (SecurityTransaction*) transaction;
+	if(d_shares != sectrans->shares()) return false;
+	if(d_share_value != sectrans->shareValue()) return false;
+	if(o_security != sectrans->security()) return false;
+	return true;
+}
+
 double SecurityTransaction::shares() const {return d_shares;}
-double SecurityTransaction::shareValue() const {return d_value / d_shares;}
-void SecurityTransaction::setSharesAndShareValue(double new_shares, double new_share_value) {
-	d_shares = new_shares; d_value = new_share_value * d_shares;
-	if(o_security) o_security->setQuotation(d_date, new_share_value, true);
+double SecurityTransaction::shareValue() const {return d_share_value;}
+void SecurityTransaction::setShares(double new_shares) {
+	d_shares = new_shares;
 }
-void SecurityTransaction::setSharesAndValue(double new_shares, double new_value) {
-	d_shares = new_shares; d_value = new_value;
-	if(o_security) o_security->setQuotation(d_date, shareValue(), true);
-}
-void SecurityTransaction::setValueAndShareValue(double new_value, double new_share_value) {
-	d_shares = new_value / new_share_value; d_value = new_value;
-	if(o_security) o_security->setQuotation(d_date, new_share_value, true);
+void SecurityTransaction::setShareValue(double new_share_value) {
+	d_share_value = new_share_value;
+	if(o_security) o_security->setQuotation(d_date, d_share_value, true);
 }
 Account *SecurityTransaction::fromAccount() const {return o_security->account();}
 Account *SecurityTransaction::toAccount() const {return o_security->account();}
@@ -264,10 +324,11 @@ Security *SecurityTransaction::security() const {return o_security;}
 void SecurityTransaction::save(QDomElement *e) const {
 	Transaction::save(e);
 	e->setAttribute("shares", QString::number(d_shares, 'f', o_security->decimals()));
+	e->setAttribute("sharevalue", QString::number(d_share_value, 'f', KGlobal::locale()->fracDigits()));
 	e->setAttribute("security", o_security->id());
 }
 
-SecurityBuy::SecurityBuy(Security *parent_security, double initial_value, double initial_shares, QDate initial_date, Account *from_account, QString initial_comment) : SecurityTransaction(parent_security, initial_value, initial_shares, initial_date, initial_comment) {
+SecurityBuy::SecurityBuy(Security *parent_security, double initial_value, double initial_shares, double initial_share_value, QDate initial_date, Account *from_account, QString initial_comment) : SecurityTransaction(parent_security, initial_value, initial_shares, initial_share_value, initial_date, initial_comment) {
 	setAccount(from_account);
 	if(o_security) {
 		setDescription(QString(i18n("Security: %1 (bought)")).arg(o_security->name()));
@@ -276,6 +337,7 @@ SecurityBuy::SecurityBuy(Security *parent_security, double initial_value, double
 SecurityBuy::SecurityBuy(Budget *parent_budget, QDomElement *e, bool *valid) : SecurityTransaction(parent_budget, e, valid) {
 	int id_account = e->attribute("account").toInt();
 	d_value = e->attribute("cost").toDouble();
+	if(d_share_value < 0.0) d_share_value = d_value / d_shares;
 	if(budget()->assetsAccounts_id.contains(id_account)) {
 		setAccount(budget()->assetsAccounts_id[id_account]);
 	} else if(budget()->incomesAccounts_id.contains(id_account)) {
@@ -304,7 +366,7 @@ void SecurityBuy::save(QDomElement *e) const {
 	e->setAttribute("account", account()->id());
 }
 
-SecuritySell::SecuritySell(Security *parent_security, double initial_value, double initial_shares, QDate initial_date, Account *to_account, QString initial_comment) : SecurityTransaction(parent_security, initial_value, initial_shares, initial_date, initial_comment) {
+SecuritySell::SecuritySell(Security *parent_security, double initial_value, double initial_shares, double initial_share_value, QDate initial_date, Account *to_account, QString initial_comment) : SecurityTransaction(parent_security, initial_value, initial_shares, initial_share_value, initial_date, initial_comment) {
 	setAccount(to_account);
 	if(o_security) {
 		setDescription(QString(i18n("Security: %1 (sold)")).arg(o_security->name()));
@@ -313,6 +375,7 @@ SecuritySell::SecuritySell(Security *parent_security, double initial_value, doub
 SecuritySell::SecuritySell(Budget *parent_budget, QDomElement *e, bool *valid) : SecurityTransaction(parent_budget, e, valid) {
 	int id_account = e->attribute("account").toInt();
 	d_value = e->attribute("income").toDouble();
+	if(d_share_value < 0.0) d_share_value = d_value / d_shares;
 	if(budget()->assetsAccounts_id.contains(id_account)) {
 		setAccount(budget()->assetsAccounts_id[id_account]);
 	} else if(budget()->expensesAccounts_id.contains(id_account)) {
@@ -515,9 +578,14 @@ bool ScheduledTransaction::isOneTimeTransaction() const {
 	return !o_rec || o_rec->firstOccurrence() == o_rec->lastOccurrence();
 }
 void ScheduledTransaction::setDate(const QDate &newdate) {
-	if(o_rec) o_rec->setStartDate(newdate);
-	if(o_trans) o_trans->setDate(o_rec->startDate());
-	o_budget->scheduledTransactionDateModified(this);
+	if(o_rec) {
+		o_rec->setStartDate(newdate);
+		if(o_trans) o_trans->setDate(o_rec->startDate());
+		o_budget->scheduledTransactionDateModified(this);
+	} else if(o_trans && newdate != o_trans->date()) {
+		o_trans->setDate(newdate);
+		o_budget->scheduledTransactionDateModified(this);
+	}
 }
 void ScheduledTransaction::addException(const QDate &exceptiondate) {
 	if(!o_rec) return;
@@ -528,3 +596,190 @@ void ScheduledTransaction::addException(const QDate &exceptiondate) {
 		o_rec->addException(exceptiondate);
 	}
 }
+
+SplitTransaction::SplitTransaction(Budget *parent_budget, QDate initial_date, AssetsAccount *initial_account, QString initial_description) : o_budget(parent_budget), d_date(initial_date), o_account(initial_account), s_description(initial_description.stripWhiteSpace()) {}
+SplitTransaction::SplitTransaction(Budget *parent_budget, QDomElement *e, bool *valid) : o_budget(parent_budget) {
+	o_account = NULL;
+	d_date = QDate::fromString(e->attribute("date"), Qt::ISODate);
+	s_description = e->attribute("description", emptystr).stripWhiteSpace();
+	s_comment = e->attribute("comment", emptystr);
+	int id = e->attribute("account").toInt();
+	if(d_date.isValid() && parent_budget->assetsAccounts_id.contains(id)) {
+		o_account = parent_budget->assetsAccounts_id[id];
+		for(QDomNode n = e->firstChild(); !n.isNull(); n = n.nextSibling()) {
+			if(n.isElement()) {
+				QDomElement e2 = n.toElement();
+				if(e2.tagName() == "transaction") {
+					QString type = e2.attribute("type");
+					e2.setAttribute("date", d_date.toString(Qt::ISODate));
+					bool valid2 = true;
+					Transaction *trans = NULL;
+					if(type == "expense") {
+						e2.setAttribute("from", id);
+						trans = new Expense(parent_budget, &e2, &valid2);
+					} else if(type == "income") {
+						e2.setAttribute("to", id);
+						trans = new Income(parent_budget, &e2, &valid2);
+					} else if(type == "transfer") {
+						if(e2.hasAttribute("to")) e2.setAttribute("from", id);
+						else e2.setAttribute("to", id);
+						trans = new Transfer(parent_budget, &e2, &valid2);
+					}
+					if(!valid2) {
+						delete trans;
+						trans = NULL;
+					}
+					if(trans) {
+						trans->setParentSplit(this);
+						splits.push_back(trans);
+					}
+				}
+			}
+		} 
+	} else {
+		if(valid) *valid =false;
+	}	
+}
+SplitTransaction::SplitTransaction() : o_budget(NULL), o_account(NULL) {}
+SplitTransaction::~SplitTransaction() {
+	clear();
+}
+
+double SplitTransaction::value() const {
+	double d_value = 0.0;
+	QValueVector<Transaction*>::size_type c = splits.size();
+	for(QValueVector<Transaction*>::size_type i = 0; i < c; i++) {
+		Transaction *trans = splits[i];
+		if(trans->fromAccount() == o_account) d_value -= trans->value();
+		if(trans->toAccount() == o_account) d_value += trans->value();
+	}
+	return d_value;
+}
+void SplitTransaction::addTransaction(Transaction *trans) {
+	trans->setDate(d_date);
+	switch(trans->type()) {
+		case TRANSACTION_TYPE_EXPENSE: {
+			((Expense*) trans)->setFrom(o_account);
+			break;
+		}
+		case TRANSACTION_TYPE_INCOME: {
+			((Income*) trans)->setTo(o_account);
+			break;
+		}
+		case TRANSACTION_TYPE_TRANSFER: {
+			if(!((Transfer*) trans)->from()) {
+				((Transfer*) trans)->setFrom(o_account);
+			} else {
+				((Transfer*) trans)->setTo(o_account);
+			}
+			break;
+		}
+		case TRANSACTION_TYPE_SECURITY_BUY: {}
+		case TRANSACTION_TYPE_SECURITY_SELL: {
+			((SecurityTransaction*) trans)->setAccount(o_account);
+			break;
+		}
+	}
+	splits.push_back(trans);
+	trans->setParentSplit(this);
+}
+void SplitTransaction::removeTransaction(Transaction *trans, bool keep) {
+	QValueVector<Transaction*>::iterator it_e = splits.end();
+	for(QValueVector<Transaction*>::iterator it = splits.begin(); it != it_e; ++it) {
+		if(*it == trans) {
+			splits.erase(it);
+			break;
+		}
+	}
+	trans->setParentSplit(NULL);
+	o_budget->removeTransaction(trans, keep);
+}
+void SplitTransaction::clear(bool keep) {
+	QValueVector<Transaction*>::iterator it_e = splits.end();
+	for(QValueVector<Transaction*>::iterator it = splits.begin(); it != it_e; ++it) {
+		(*it)->setParentSplit(NULL);
+		if(!keep) o_budget->removeTransaction(*it);
+	}
+	splits.clear();
+}
+const QDate &SplitTransaction::date() const {return d_date;}
+void SplitTransaction::setDate(QDate new_date) {
+	QDate old_date = d_date; d_date = new_date; o_budget->splitTransactionDateModified(this, old_date);
+	QValueVector<Transaction*>::size_type c = splits.count();
+	for(QValueVector<Transaction*>::size_type i = 0; i < c; i++) {
+		splits[i]->setDate(d_date);
+	}
+}
+const QString &SplitTransaction::description() const {return s_description;}
+void SplitTransaction::setDescription(QString new_description) {s_description = new_description.stripWhiteSpace();}
+const QString &SplitTransaction::comment() const {return s_comment;}
+void SplitTransaction::setComment(QString new_comment) {s_comment = new_comment;}
+AssetsAccount *SplitTransaction::account() const {return o_account;}
+void SplitTransaction::setAccount(AssetsAccount *new_account) {
+	QValueVector<Transaction*>::size_type c = splits.count();
+	for(QValueVector<Transaction*>::size_type i = 0; i < c; i++) {
+		Transaction *trans = splits[i];
+		switch(trans->type()) {
+			case TRANSACTION_TYPE_EXPENSE: {
+				((Expense*) trans)->setFrom(new_account);
+				break;
+			}
+			case TRANSACTION_TYPE_INCOME: {
+				((Income*) trans)->setTo(new_account);
+				break;
+			}
+			case TRANSACTION_TYPE_TRANSFER: {
+				if(((Transfer*) trans)->from() == o_account) {
+					((Transfer*) trans)->setFrom(new_account);
+				} else {
+					((Transfer*) trans)->setTo(new_account);
+				}
+				break;
+			}
+			case TRANSACTION_TYPE_SECURITY_BUY: {}
+			case TRANSACTION_TYPE_SECURITY_SELL: {
+				((SecurityTransaction*) trans)->setAccount(new_account);
+				break;
+			}
+		}
+	}
+	o_account = new_account;
+}
+Budget *SplitTransaction::budget() const {return o_budget;}
+void SplitTransaction::save(QDomElement *e) const {
+	e->setAttribute("date", d_date.toString(Qt::ISODate));
+	e->setAttribute("account", o_account->id());
+	if(!s_description.isEmpty()) e->setAttribute("description", s_description);
+	if(!s_comment.isEmpty())  e->setAttribute("comment", s_comment);
+	QValueVector<Transaction*>::size_type c = splits.count();
+	for(QValueVector<Transaction*>::size_type i = 0; i < c; i++) {
+		QDomElement e2 = e->ownerDocument().createElement("transaction");
+		Transaction *trans = splits[i];
+		trans->save(&e2);
+		e2.removeAttribute("date");
+		switch(trans->type()) {
+			case TRANSACTION_TYPE_TRANSFER: {
+				e2.setAttribute("type", "transfer");
+				if(((Transfer*) trans)->from() == o_account) {
+					e2.removeAttribute("from");
+				} else {
+					e2.removeAttribute("to");
+				}				
+				break;
+			}
+			case TRANSACTION_TYPE_INCOME: {
+				e2.setAttribute("type", "income");
+				e2.removeAttribute("to");
+				break;
+			}
+			case TRANSACTION_TYPE_EXPENSE: {
+				e2.setAttribute("type", "expense");
+				e2.removeAttribute("from");
+				break;
+			}
+			default: {}
+		}	
+		e->appendChild(e2);
+	}
+}
+
