@@ -203,6 +203,15 @@ TransactionFilterWidget::~TransactionFilterWidget() {
 	delete group;
 }
 
+void TransactionFilterWidget::currentDateChanged(const QDate &olddate, const QDate &newdate) {
+	if(olddate == to_date) {
+		dateToEdit->blockSignals(true);
+		dateToEdit->setDate(newdate);
+		dateToEdit->blockSignals(false);
+		toChanged(newdate);
+	}
+}
+
 void TransactionFilterWidget::clearFilter() {
 	dateFromButton->blockSignals(true);
 	dateToEdit->blockSignals(true);
