@@ -97,6 +97,8 @@ TransactionListWidget::TransactionListWidget(bool extra_parameters, int transact
 	current_value = 0.0;
 	current_quantity = 0.0;
 
+	selected_trans = NULL;
+
 	listPopupMenu = NULL;
 	
 	QVBoxLayout *transactionsLayout = new QVBoxLayout(this, 0, 6);
@@ -163,7 +165,7 @@ TransactionListWidget::TransactionListWidget(bool extra_parameters, int transact
 	tabs->setMargin(6);
 	tabs->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
 
-	editWidget = new TransactionEditWidget(true, b_extra, transtype, NULL, SECURITY_SHARES_AND_QUOTATION, budget, this);
+	editWidget = new TransactionEditWidget(true, b_extra, transtype, false, false, NULL, SECURITY_SHARES_AND_QUOTATION, budget, this);
 	editInfoLabel = new QLabel(QString::null, editWidget->bottomWidget());
 	KButtonBox *buttons = new KButtonBox(editWidget->bottomWidget());
 	buttons->addStretch();
@@ -198,6 +200,8 @@ TransactionListWidget::TransactionListWidget(bool extra_parameters, int transact
 	connect(filterWidget, SIGNAL(fromActivated(int)), this, SLOT(filterFromActivated(int)));
 	connect(transactionsView, SIGNAL(selectionChanged()), this, SLOT(transactionSelectionChanged()));
 	connect(transactionsView, SIGNAL(contextMenu(KListView*, QListViewItem*, const QPoint&)), this, SLOT(popupListMenu(KListView*, QListViewItem*, const QPoint&)));
+	connect(transactionsView, SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int)), this, SLOT(transactionExecuted(QListViewItem*)));
+	connect(transactionsView, SIGNAL(returnPressed(QListViewItem*)), this, SLOT(transactionExecuted(QListViewItem*)));
 
 }
 
@@ -207,6 +211,18 @@ TransactionListWidget::TransactionListWidget(bool extra_parameters, int transact
 QSize TransactionListWidget::minimumSizeHint() const {return QWidget::minimumSizeHint();}
 QSize TransactionListWidget::sizeHint() const {return minimumSizeHint();}
 
+void TransactionListWidget::currentDateChanged(const QDate &olddate, const QDate &newdate) {
+#if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
+#else
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems();
+#endif
+	if(selection.count() == 0) editWidget->currentDateChanged(olddate, newdate);
+	filterWidget->currentDateChanged(olddate, newdate);
+}
+void TransactionListWidget::transactionExecuted(QListViewItem*) {
+	editTransaction();
+}
 void TransactionListWidget::updateStatistics() {
 	int i_count_frac = 0;
 	double intpart = 0.0;
@@ -218,16 +234,21 @@ void TransactionListWidget::popupListMenu(KListView*, QListViewItem*, const QPoi
 	if(!listPopupMenu) {
 		listPopupMenu = new QPopupMenu(this);
 		switch(transtype) {
-			case TRANSACTION_TYPE_EXPENSE: {mainWin->ActionNewExpense->plug(listPopupMenu); break;}
-			case TRANSACTION_TYPE_INCOME: {mainWin->ActionNewIncome->plug(listPopupMenu); break;}
+			case TRANSACTION_TYPE_EXPENSE: {mainWin->ActionNewExpense->plug(listPopupMenu); mainWin->ActionNewRefund->plug(listPopupMenu); break;}
+			case TRANSACTION_TYPE_INCOME: {mainWin->ActionNewIncome->plug(listPopupMenu); mainWin->ActionNewRepayment->plug(listPopupMenu); break;}
 			case TRANSACTION_TYPE_TRANSFER: {mainWin->ActionNewTransfer->plug(listPopupMenu); break;}
 		}
+		mainWin->ActionNewSplitTransaction->plug(listPopupMenu);
 		listPopupMenu->insertSeparator();
 		mainWin->ActionEditTransaction->plug(listPopupMenu);
 		mainWin->ActionEditScheduledTransaction->plug(listPopupMenu);
+		mainWin->ActionEditSplitTransaction->plug(listPopupMenu);
+		mainWin->ActionJoinTransactions->plug(listPopupMenu);
+		mainWin->ActionSplitUpTransaction->plug(listPopupMenu);
 		listPopupMenu->insertSeparator();
 		mainWin->ActionDeleteTransaction->plug(listPopupMenu);
 		mainWin->ActionDeleteScheduledTransaction->plug(listPopupMenu);
+		mainWin->ActionDeleteSplitTransaction->plug(listPopupMenu);
 	}
 	listPopupMenu->popup(p);
 }
@@ -271,7 +292,8 @@ bool TransactionListWidget::exportList(QTextStream &outf, int fileformat) {
 			outf << "<th>" << htmlize_string(transactionsView->columnText(1)) << "</th>";
 			outf << "<th>" << htmlize_string(transactionsView->columnText(2)) << "</th>";
 			outf << "<th>" << htmlize_string(transactionsView->columnText(3)) << "</th>";
-			outf << "<th>" << htmlize_string(transactionsView->columnText(4)) << "</th>";			
+			outf << "<th>" << htmlize_string(transactionsView->columnText(4)) << "</th>";
+			if(comments_col == 6) outf << "<th>" << htmlize_string(i18n("Quantity")) << "</th>";
 			outf << "<th>" << htmlize_string(transactionsView->columnText(5)) << "</th>";
 			if(comments_col == 6) outf << "<th>" << htmlize_string(transactionsView->columnText(6)) << "</th>";
 			outf << "\n";
@@ -283,10 +305,14 @@ bool TransactionListWidget::exportList(QTextStream &outf, int fileformat) {
 				Transaction *trans = i->transaction();
 				outf << "\t\t\t\t<tr>" << '\n';
 				outf << "\t\t\t\t\t<td nowrap align=\"right\">" << htmlize_string(KGlobal::locale()->formatDate(trans->date(), true)) << "</td>";
-				outf << "<td>" << htmlize_string(i->text(1)) << "</td>";
+				outf << "<td>" << htmlize_string(trans->description()) << "</td>";
 				outf << "<td nowrap align=\"right\">" << htmlize_string(i->text(2)) << "</td>";
 				outf << "<td align=\"center\">" << htmlize_string(i->text(3)) << "</td>";
 				outf << "<td align=\"center\">" << htmlize_string(i->text(4)) << "</td>";
+				int i_count_frac = 0;
+				double intpart = 0.0;
+				if(modf(trans->quantity(), &intpart) != 0.0) i_count_frac = 2;
+				if(comments_col == 6) outf << "<td>" << htmlize_string(KGlobal::locale()->formatNumber(trans->quantity(), i_count_frac)) << "</td>";
 				outf << "<td>" << htmlize_string(i->text(5)) << "</td>";
 				if(comments_col == 6) outf << "<td>" << htmlize_string(i->text(6)) << "</td>";
 				outf << "\n";
@@ -299,27 +325,27 @@ bool TransactionListWidget::exportList(QTextStream &outf, int fileformat) {
 			int i_count_frac = 0;
 			double intpart = 0.0;
 			if(modf(current_quantity, &intpart) != 0.0) i_count_frac = 2;
-			outf << i18n("Quantity:") << " " << KGlobal::locale()->formatNumber(current_quantity, i_count_frac);
+			outf << htmlize_string(i18n("Quantity:")) << " " << htmlize_string(KGlobal::locale()->formatNumber(current_quantity, i_count_frac));
 			outf << ", ";
 			switch(transtype) {
 				case TRANSACTION_TYPE_EXPENSE: {
-					outf << i18n("Total cost:") << " ";
+					outf << htmlize_string(i18n("Total cost:")) << " ";
 					break;
 				}
 				case TRANSACTION_TYPE_INCOME: {
-					outf << i18n("Total income:") << " ";
+					outf << htmlize_string(i18n("Total income:")) << " ";
 					break;
 				}
 				default: {
-					outf << i18n("Total amount:") << " ";
+					outf << htmlize_string(i18n("Total amount:")) << " ";
 					break;
 				}
 			}
-			outf << KGlobal::locale()->formatMoney(current_value);
+			outf << htmlize_string(KGlobal::locale()->formatMoney(current_value));
 			outf << ", ";
-			outf << i18n("Monthly average:") << " " << KGlobal::locale()->formatMoney(current_value == 0.0 ? current_value : current_value / filterWidget->countMonths());
+			outf << htmlize_string(i18n("Average:")) << " " << htmlize_string(KGlobal::locale()->formatMoney(current_value == 0.0 ? current_value : current_value / current_quantity));
 			outf << ", ";
-			outf << i18n("Daily average:") << " " << KGlobal::locale()->formatMoney(current_value == 0.0 ? current_value : current_value / filterWidget->countDays());
+			outf << htmlize_string(i18n("Monthly average:")) << " " << htmlize_string(KGlobal::locale()->formatMoney(current_value == 0.0 ? current_value : current_value / filterWidget->countMonths()));
 			outf << "</div>\n";
 			outf << "\t</body>" << '\n';
 			outf << "</html>" << '\n';
@@ -327,13 +353,17 @@ bool TransactionListWidget::exportList(QTextStream &outf, int fileformat) {
 		}
 		case 'c': {
 			outf.setEncoding(QTextStream::Locale);
-			outf << "\"" << transactionsView->columnText(0) << "\",\"" << transactionsView->columnText(1) << "\",\"" << transactionsView->columnText(2) << "\",\"" << transactionsView->columnText(3) << "\",\"" << transactionsView->columnText(4) << "\",\"" << transactionsView->columnText(5);
-			if(comments_col == 6) outf << "\",\"" << transactionsView->columnText(5);
+			outf << "\"" << transactionsView->columnText(0) << "\",\"" << transactionsView->columnText(1) << "\",\"" << transactionsView->columnText(2) << "\",\"" << transactionsView->columnText(3) << "\",\"" << transactionsView->columnText(4);
+			if(comments_col == 6) outf << "\",\"" << i18n("Quantity");
+			outf << "\",\"" << transactionsView->columnText(5);
+			if(comments_col == 6) outf << "\",\"" << transactionsView->columnText(6);
 			outf << "\"\n";
 			TransactionListViewItem *i = (TransactionListViewItem*) transactionsView->firstChild();
 			while(i) {
 				Transaction *trans = i->transaction();
-				outf << "\"" << KGlobal::locale()->formatDate(trans->date(), true) << "\",\"" << i->text(1) << "\",\"" << i->text(2) << "\",\"" << i->text(3) << "\",\"" << i->text(4) << "\",\"" << i->text(5);
+				outf << "\"" << KGlobal::locale()->formatDate(trans->date(), true) << "\",\"" << trans->description() << "\",\"" << i->text(2) << "\",\"" << i->text(3) << "\",\"" << i->text(4);
+				if(comments_col == 6) outf << "\",\"" << KGlobal::locale()->formatNumber(trans->quantity(), 2);
+				outf << "\",\"" << i->text(5);
 				if(comments_col == 6) outf << "\",\"" << i->text(6);
 				outf << "\"\n";
 				i = (TransactionListViewItem*) i->nextSibling();
@@ -349,14 +379,13 @@ bool TransactionListWidget::exportList(QTextStream &outf, int fileformat) {
 void TransactionListWidget::transactionsReset() {
 	filterWidget->transactionsReset();
 	editWidget->transactionsReset();
+	editClear();
 	filterTransactions();
 }
 void TransactionListWidget::addTransaction() {
 	Transaction *trans = editWidget->createTransaction();
 	if(!trans) return;
-	transactionsView->selectAll(false);
-	editWidget->setTransaction(NULL);
-	editInfoLabel->setText(QString::null);
+	editClear();
 	ScheduledTransaction *strans = NULL;
 	if(trans->date() > QDate::currentDate()) {
 		strans = new ScheduledTransaction(budget, trans, NULL);
@@ -380,6 +409,19 @@ void TransactionListWidget::editScheduledTransaction() {
 		}
 	}
 }
+void TransactionListWidget::editSplitTransaction() {
+#if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
+#else
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems();
+#endif
+	if(selection.count() >= 1) {
+		TransactionListViewItem *i = (TransactionListViewItem*) selection.getFirst();
+		if(i->transaction()->parentSplit()) {
+			if(mainWin->editSplitTransaction(i->transaction()->parentSplit())) editClear();
+		}
+	}
+}
 void TransactionListWidget::editTransaction() {
 #if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
 	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
@@ -390,16 +432,15 @@ void TransactionListWidget::editTransaction() {
 		TransactionListViewItem *i = (TransactionListViewItem*) selection.getFirst();
 		if(i->scheduledTransaction()) {
 			if(i->scheduledTransaction()->isOneTimeTransaction()) {
-				mainWin->editScheduledTransaction(i->scheduledTransaction());
+				if(mainWin->editScheduledTransaction(i->scheduledTransaction())) editClear();
 			} else {
-				mainWin->editOccurrence(i->scheduledTransaction(), i->date());
+				if(mainWin->editOccurrence(i->scheduledTransaction(), i->date())) transactionSelectionChanged();
 			}
 		} else {
-			mainWin->editTransaction(i->transaction());
+			if(mainWin->editTransaction(i->transaction())) transactionSelectionChanged();
 		}
-		transactionsView->selectAll(false);
 	} else if(selection.count() > 1) {
-		bool warned1 = false, warned2 = false, warned3 = false;
+		bool warned1 = false, warned2 = false, warned3 = false, warned4 = false;
 		MultipleTransactionsEditDialog *dialog = new MultipleTransactionsEditDialog(b_extra, transtype, budget, this);
 		TransactionListViewItem *i = (TransactionListViewItem*) transactionsView->currentItem();
 		if(!i->isSelected()) i = (TransactionListViewItem*) selection.first();
@@ -416,6 +457,7 @@ void TransactionListWidget::editTransaction() {
 			if(!comptrans) {
 				comptrans = i->transaction();
 				compdate = i->date();
+				if(i->transaction()->parentSplit()) equal_date = false;
 				if(i->transaction()->type() != TRANSACTION_TYPE_EXPENSE && i->transaction()->type() != TRANSACTION_TYPE_INCOME) equal_payee = false;
 				if(i->transaction()->type() == TRANSACTION_TYPE_INCOME) {
 					compcat = ((Income*) i->transaction())->category();
@@ -430,13 +472,13 @@ void TransactionListWidget::editTransaction() {
 					}
 				}
 			} else {
-				if(equal_date && compdate != i->date()) {
+				if(equal_date && (compdate != i->date() || i->transaction()->parentSplit())) {
 					equal_date = false;
 				}
 				if(equal_description && (i->transaction()->type() == TRANSACTION_TYPE_SECURITY_BUY || i->transaction()->type() == TRANSACTION_TYPE_SECURITY_SELL || comptrans->description() != i->transaction()->description())) {
 					equal_description = false;
 				}
-				if(equal_payee && (i->transaction()->type() != comptrans->type() || (comptrans->type() == TRANSACTION_TYPE_EXPENSE && ((Expense*) comptrans)->payee() != ((Expense*) i->transaction())->payee()) || (comptrans->type() == TRANSACTION_TYPE_EXPENSE && ((Income*) comptrans)->payer() != ((Income*) i->transaction())->payer()))) {
+				if(equal_payee && (i->transaction()->type() != comptrans->type() || (comptrans->type() == TRANSACTION_TYPE_EXPENSE && ((Expense*) comptrans)->payee() != ((Expense*) i->transaction())->payee()) || (comptrans->type() == TRANSACTION_TYPE_INCOME && ((Income*) comptrans)->payer() != ((Income*) i->transaction())->payer()))) {
 					equal_payee = false;
 				}
 				if(equal_value && (i->transaction()->type() == TRANSACTION_TYPE_SECURITY_BUY || i->transaction()->type() == TRANSACTION_TYPE_SECURITY_SELL || comptrans->value() != i->transaction()->value())) {
@@ -488,6 +530,12 @@ void TransactionListWidget::editTransaction() {
 						warned3 = true;
 					}
 				}
+				if(!warned4 && i->transaction()->parentSplit()) {
+					if(dialog->dateButton->isChecked()) {
+						KMessageBox::error(this, i18n("Cannot change date of transactions that are part of a split transaction."));
+						warned4 = true;
+					}
+				}
 				ScheduledTransaction *strans = i->scheduledTransaction();
 				if(strans && strans->isOneTimeTransaction() && (date.isNull() || date == strans->transaction()->date())) {
 					ScheduledTransaction *old_strans = strans->copy();
@@ -526,7 +574,7 @@ void TransactionListWidget::editTransaction() {
 					Transaction *trans = i->transaction();
 					Transaction *oldtrans = trans->copy();
 					if(dialog->modifyTransaction(trans)) {
-						if(future) {
+						if(future && !trans->parentSplit()) {
 							budget->removeTransaction(trans, true);
 							mainWin->transactionRemoved(trans);
 							strans = new ScheduledTransaction(budget, trans, NULL);
@@ -540,8 +588,9 @@ void TransactionListWidget::editTransaction() {
 				}
 				i = (TransactionListViewItem*) selection.next();
 			}
+			transactionSelectionChanged();
 		}
-		delete dialog;
+		dialog->deleteLater();
 	}
 }
 void TransactionListWidget::modifyTransaction() {
@@ -562,16 +611,38 @@ void TransactionListWidget::modifyTransaction() {
 		return;
 	}
 	if(!editWidget->validValues()) return;
-	ScheduledTransaction *oldstrans = NULL;
 	if(i->scheduledTransaction()) {
-		if(i->scheduledTransaction()->isOneTimeTransaction() && i->scheduledTransaction()->firstOccurrence() > QDate::currentDate()) {
-			oldstrans = i->scheduledTransaction()->copy();
+		if(i->scheduledTransaction()->isOneTimeTransaction()) {
+			Transaction *trans = i->scheduledTransaction()->transaction()->copy();
+			if(!editWidget->modifyTransaction(trans)) {
+				delete trans;
+				return;
+			}
+			mainWin->removeScheduledTransaction(i->scheduledTransaction());
+			ScheduledTransaction *strans = NULL;
+			if(trans->date() > QDate::currentDate()) {
+				strans = new ScheduledTransaction(budget, trans, NULL);
+				budget->addScheduledTransaction(strans);
+			} else {
+				budget->addTransaction(trans);
+			}
+			if(strans) mainWin->scheduledTransactionAdded(strans);
+			else mainWin->transactionAdded(trans);
+			TransactionListViewItem *i = (TransactionListViewItem*) transactionsView->firstChild();
+			while(i) {
+				if(i->transaction() == trans) {
+					transactionsView->setCurrentItem(i);
+					transactionsView->setSelected(i, true);
+					break;
+				}
+				i = (TransactionListViewItem*) i->nextSibling();
+			}
+			return;
 		} else {
 			if(editWidget->validValues()) {
 				ScheduledTransaction *curstranscopy = i->scheduledTransaction();
 				QDate curdate_copy = i->date();
-				addTransaction();
-				oldstrans = curstranscopy->copy();
+				ScheduledTransaction *oldstrans = curstranscopy->copy();
 				curstranscopy->addException(curdate_copy);
 				mainWin->scheduledTransactionModified(curstranscopy, oldstrans);
 				delete oldstrans;
@@ -579,19 +650,22 @@ void TransactionListWidget::modifyTransaction() {
 			return;
 		}
 	}
-	if(i->scheduledTransaction()) {
-		if(editWidget->modifyTransaction(i->transaction())) {
-			i->scheduledTransaction()->setDate(i->transaction()->date());
-			mainWin->scheduledTransactionModified(i->scheduledTransaction(), oldstrans);
-		}
-		delete oldstrans;
-	} else if(editWidget->date() > QDate::currentDate()) {
+	if(editWidget->date() > QDate::currentDate()) {
 		Transaction *newtrans = i->transaction()->copy();
 		if(editWidget->modifyTransaction(newtrans)) {
 			ScheduledTransaction *strans = new ScheduledTransaction(budget, newtrans, NULL);
 			removeTransaction();
 			budget->addScheduledTransaction(strans);
 			mainWin->scheduledTransactionAdded(strans);
+			TransactionListViewItem *i = (TransactionListViewItem*) transactionsView->firstChild();
+			while(i) {
+				if(i->scheduledTransaction() == strans) {
+					transactionsView->setCurrentItem(i);
+					transactionsView->setSelected(i, true);
+					break;
+				}
+				i = (TransactionListViewItem*) i->nextSibling();
+			}
 		} else {
 			delete newtrans;
 		}
@@ -613,9 +687,71 @@ void TransactionListWidget::removeScheduledTransaction() {
 		TransactionListViewItem *i = (TransactionListViewItem*) selection.getFirst();
 		if(i->scheduledTransaction()) {
 			mainWin->removeScheduledTransaction(i->scheduledTransaction());
+			editClear();
 		}
 	}
 }
+void TransactionListWidget::removeSplitTransaction() {
+#if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
+#else
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems();
+#endif
+	if(selection.count() >= 1) {
+		TransactionListViewItem *i = (TransactionListViewItem*) selection.getFirst();
+		if(i->transaction()->parentSplit()) {
+			SplitTransaction *split = i->transaction()->parentSplit();		
+			if(!KMessageBox::warningContinueCancel(this, i18n("Are you sure you want to delete all (%1) transactions in the selected split transaction?").arg(split->splits.count()))) {
+				return;
+			}
+			budget->removeSplitTransaction(split, true);
+			mainWin->splitTransactionRemoved(split);
+			delete split;
+			editClear();
+		}
+	}
+}
+void TransactionListWidget::splitUpTransaction() {
+#if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
+#else
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems();
+#endif
+	if(selection.count() >= 1) {
+		TransactionListViewItem *i = (TransactionListViewItem*) selection.getFirst();
+		if(i->transaction()->parentSplit()) {
+			mainWin->splitUpTransaction(i->transaction()->parentSplit());
+			transactionSelectionChanged();
+		}
+	}
+}
+void TransactionListWidget::joinTransactions() {
+#if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
+#else
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems();
+#endif	
+	SplitTransaction *split = NULL;
+	TransactionListViewItem *i = (TransactionListViewItem*) selection.first();
+	while(i) {
+		Transaction *trans = i->transaction();
+		if(trans->type() != TRANSACTION_TYPE_SECURITY_BUY && trans->type() != TRANSACTION_TYPE_SECURITY_SELL && (trans->type() != TRANSACTION_TYPE_INCOME || !((Income*) trans)->security())) {
+			if(!split) {
+				if(trans->fromAccount()->type() == ACCOUNT_TYPE_ASSETS) split = new SplitTransaction(budget, i->transaction()->date(), (AssetsAccount*) trans->fromAccount());
+				else split = new SplitTransaction(budget, i->transaction()->date(), (AssetsAccount*) trans->fromAccount());
+			}
+			split->splits.push_back(trans);
+		}
+		i = (TransactionListViewItem*) selection.next();
+	}
+	if(!split) return;
+	if(mainWin->editSplitTransaction(split)) {
+		editClear();
+	} else {
+		split->clear(true);
+		delete split;
+	}
+}
 void TransactionListWidget::removeTransaction() {
 #if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
 	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
@@ -657,7 +793,14 @@ void TransactionListWidget::addModifyTransaction() {
 
 void TransactionListWidget::appendFilterTransaction(Transaction *trans, bool update_total_value) {
 	if(!filterWidget->filterTransaction(trans)) {
-		QListViewItem *i = new TransactionListViewItem(trans->date(), trans, NULL, transactionsView, KGlobal::locale()->formatDate(trans->date(), true), trans->description(), KGlobal::locale()->formatMoney(trans->value()));
+		QListViewItem *i = new TransactionListViewItem(trans->date(), trans, NULL, transactionsView, KGlobal::locale()->formatDate(trans->date(), true), QString::null, KGlobal::locale()->formatMoney(trans->value()));
+		if(trans == selected_trans) {
+			transactionsView->blockSignals(true);
+			transactionsView->setSelected(i, true);
+			transactionsView->blockSignals(false);
+		}
+		if(trans->parentSplit()) i->setText(1, trans->description() + "*");
+		else i->setText(1, trans->description());
 		i->setText(from_col, trans->fromAccount()->name());
 		i->setText(to_col, trans->toAccount()->name());
 		if(comments_col == 6 && trans->type() == TRANSACTION_TYPE_EXPENSE) i->setText(5, ((Expense*) trans)->payee());
@@ -686,8 +829,9 @@ void TransactionListWidget::appendFilterScheduledTransaction(ScheduledTransactio
 		Transaction *trans = strans->transaction();
 		while(!date.isNull() && date <= enddate) {
 			QListViewItem *i = NULL;
-			if(strans->recurrence()) i = new TransactionListViewItem(date, trans, strans, transactionsView, KGlobal::locale()->formatDate(date, true) + "*", trans->description(), KGlobal::locale()->formatMoney(trans->value()));
-			else i = new TransactionListViewItem(date, trans, strans, transactionsView, KGlobal::locale()->formatDate(date, true), trans->description(), KGlobal::locale()->formatMoney(trans->value()));
+			i = new TransactionListViewItem(date, trans, strans, transactionsView, QString::null, trans->description(), KGlobal::locale()->formatMoney(trans->value()));
+			if(strans->recurrence()) i->setText(0, KGlobal::locale()->formatDate(date, true) + "**");
+			else i->setText(0, KGlobal::locale()->formatDate(date, true));
 			i->setText(from_col, trans->fromAccount()->name());
 			i->setText(to_col, trans->toAccount()->name());
 			if(comments_col == 6 && trans->type() == TRANSACTION_TYPE_EXPENSE) i->setText(5, ((Expense*) trans)->payee());
@@ -704,7 +848,7 @@ void TransactionListWidget::appendFilterScheduledTransaction(ScheduledTransactio
 	}
 }
 
-void TransactionListWidget::onTransactionAdded(Transaction *trans) {
+void TransactionListWidget::onTransactionAdded(Transaction *trans) {	
 	appendFilterTransaction(trans, true);
 	filterWidget->transactionAdded(trans);
 	editWidget->transactionAdded(trans);
@@ -732,13 +876,15 @@ void TransactionListWidget::onTransactionModified(Transaction *trans, Transactio
 	if(i) {
 		i->setDate(trans->date());
 		i->setText(0, KGlobal::locale()->formatDate(trans->date(), true));
-		i->setText(1, trans->description());
+		if(trans->parentSplit()) i->setText(1, trans->description() + "*");
+		else i->setText(1, trans->description());
 		i->setText(2, KGlobal::locale()->formatMoney(trans->value()));
 		i->setText(from_col, trans->fromAccount()->name());
 		i->setText(to_col, trans->toAccount()->name());
 		if(comments_col == 6 && trans->type() == TRANSACTION_TYPE_EXPENSE) i->setText(5, ((Expense*) trans)->payee());
 		else if(comments_col == 6 && trans->type() == TRANSACTION_TYPE_INCOME) i->setText(5, ((Income*) trans)->payer());
 		i->setText(comments_col, trans->comment());
+		transactionsView->sort();
 	}
 	if(oldtrans->description() != trans->description()) filterWidget->transactionModified(trans);
 	editWidget->transactionModified(trans);
@@ -763,7 +909,7 @@ void TransactionListWidget::onScheduledTransactionAdded(ScheduledTransaction *st
 	editWidget->transactionAdded(strans->transaction());
 }
 void TransactionListWidget::onScheduledTransactionModified(ScheduledTransaction *strans, ScheduledTransaction *oldstrans) {
-	TransactionListViewItem *i = (TransactionListViewItem*) transactionsView->firstChild();
+	TransactionListViewItem *i = (TransactionListViewItem*) transactionsView->firstChild();	
 	while(i) {
 		if(i->scheduledTransaction() == strans) {
 			current_value -= oldstrans->transaction()->value();
@@ -780,6 +926,11 @@ void TransactionListWidget::onScheduledTransactionModified(ScheduledTransaction
 	if(oldstrans->transaction()->description() != strans->transaction()->description()) filterWidget->transactionModified(strans->transaction());
 	editWidget->transactionModified(strans->transaction());
 }
+void TransactionListWidget::editClear() {
+	transactionsView->selectAll(false);
+	editInfoLabel->setText(QString::null);
+	editWidget->setTransaction(NULL);	
+}
 void TransactionListWidget::onScheduledTransactionRemoved(ScheduledTransaction *strans) {
 	TransactionListViewItem *i = (TransactionListViewItem*) transactionsView->firstChild();
 	while(i) {
@@ -797,6 +948,16 @@ void TransactionListWidget::onScheduledTransactionRemoved(ScheduledTransaction *
 	editWidget->transactionRemoved(strans->transaction());
 }
 void TransactionListWidget::filterTransactions() {
+#if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
+#else
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems();
+#endif
+	selected_trans = NULL;
+	if(selection.count() == 1) {
+		TransactionListViewItem *i = (TransactionListViewItem*) selection.getFirst();
+		if(!i->scheduledTransaction()) {selected_trans = i->transaction();}
+	}
 	transactionsView->clear();
 	current_value = 0.0;
 	current_quantity = 0.0;
@@ -868,6 +1029,15 @@ void TransactionListWidget::filterTransactions() {
 			break;
 		}
 	}
+	selected_trans = NULL;
+#if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
+	selection = transactionsView->selectedItems(true);
+#else
+	selection = transactionsView->selectedItems();
+#endif
+	if(selection.count() == 0) {
+		editInfoLabel->setText("");
+	}
 	updateStatistics();
 }
 
@@ -875,10 +1045,15 @@ void TransactionListWidget::currentTransactionChanged(QListViewItem *i) {
 	if(i == NULL) {
 		editWidget->setTransaction(NULL);
 		editInfoLabel->setText(QString::null);
+	} else if(((TransactionListViewItem*) i)->transaction()->parentSplit()) {
+		editWidget->setTransaction(((TransactionListViewItem*) i)->transaction());
+		SplitTransaction *split = ((TransactionListViewItem*) i)->transaction()->parentSplit();
+		if(split->description().isEmpty() || split->description().length() > 10) editInfoLabel->setText(i18n("* Part of split transaction"));
+		else editInfoLabel->setText(i18n("* Part of split (%1)").arg(split->description()));
 	} else if(((TransactionListViewItem*) i)->scheduledTransaction()) {
 		editWidget->setScheduledTransaction(((TransactionListViewItem*) i)->scheduledTransaction(), ((TransactionListViewItem*) i)->date());
 		if(((TransactionListViewItem*) i)->scheduledTransaction()->isOneTimeTransaction()) editInfoLabel->setText(QString::null);
-		else editInfoLabel->setText(i18n("* Recurring (editing occurrance)"));
+		else editInfoLabel->setText(i18n("** Recurring (editing occurrance)"));
 	} else {
 		editWidget->setTransaction(((TransactionListViewItem*) i)->transaction());
 		editInfoLabel->setText(QString::null);
@@ -899,7 +1074,7 @@ void TransactionListWidget::transactionSelectionChanged() {
 		QListViewItem *i = selection.getFirst();
 		if(selection.count() > 1) {
 			modifyButton->setText(i18n("Modify..."));
-		} else if(((TransactionListViewItem*) i)->transaction()->type() == TRANSACTION_TYPE_SECURITY_BUY || ((TransactionListViewItem*) i)->transaction()->type() == TRANSACTION_TYPE_SECURITY_SELL) {
+		} else if(((TransactionListViewItem*) i)->transaction()->parentSplit() || ((TransactionListViewItem*) i)->transaction()->type() == TRANSACTION_TYPE_SECURITY_BUY || ((TransactionListViewItem*) i)->transaction()->type() == TRANSACTION_TYPE_SECURITY_SELL) {
 			modifyButton->setText(i18n("Edit..."));
 		} else {
 			modifyButton->setText(i18n("Change"));
@@ -908,28 +1083,80 @@ void TransactionListWidget::transactionSelectionChanged() {
 		removeButton->setEnabled(true);
 		if(transactionsView->currentItem()->isSelected()) {
 			currentTransactionChanged(transactionsView->currentItem());
+		} else if(selection.count() == 1) {
+			currentTransactionChanged(i);
 		}
 	}
 	updateTransactionActions();
 }
+void TransactionListWidget::newRefundRepayment() {
+#if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
+#else
+	QPtrList<QListViewItem> selection = transactionsView->selectedItems();
+#endif
+	if(selection.count() == 1) {
+		TransactionListViewItem *i = (TransactionListViewItem*) selection.getFirst();
+		if((i->transaction()->type() == TRANSACTION_TYPE_EXPENSE && i->transaction()->value() > 0.0) || (i->transaction()->type() == TRANSACTION_TYPE_INCOME && i->transaction()->value() > 0.0 && !((Income*) i->transaction())->security())) {
+			mainWin->newRefundRepayment(i->transaction());
+		}
+	}
+}
 void TransactionListWidget::updateTransactionActions() {
 #if KDE_VERSION_MAJOR > 3 || KDE_VERSION_MINOR >= 4
 	QPtrList<QListViewItem> selection = transactionsView->selectedItems(true);
 #else
 	QPtrList<QListViewItem> selection = transactionsView->selectedItems();
 #endif
-	bool b_transaction = false, b_scheduledtransaction = false;
+	bool b_transaction = false, b_scheduledtransaction = false, b_split = false;
+	bool refundable = false, repayable = false;
 	if(selection.count() == 1) {
 		TransactionListViewItem *i = (TransactionListViewItem*) selection.getFirst();
-		b_scheduledtransaction = i->scheduledTransaction() && i->scheduledTransaction()->recurrence();
+		b_split = i->transaction() && i->transaction()->parentSplit();
+		b_scheduledtransaction = !b_split && i->scheduledTransaction() && i->scheduledTransaction()->recurrence();
 		b_transaction = !b_scheduledtransaction || !i->scheduledTransaction()->isOneTimeTransaction();
-	} else {
+		refundable = (i->transaction()->type() == TRANSACTION_TYPE_EXPENSE && i->transaction()->value() > 0.0);
+		repayable = (i->transaction()->type() == TRANSACTION_TYPE_INCOME && i->transaction()->value() > 0.0 && !((Income*) i->transaction())->security());
+	} else if(selection.count() > 1) {
 		b_transaction = true;
 	}
+	bool b_join = (selection.count() > 0);
+	b_split = b_join;
+	TransactionListViewItem *i = (TransactionListViewItem*) selection.first();
+	while(i) {
+		Transaction *trans = i->transaction();
+		if(i->scheduledTransaction() || trans->parentSplit()) {
+			b_join = false;
+			break;
+		}		
+		if(trans->type() == TRANSACTION_TYPE_SECURITY_BUY || trans->type() == TRANSACTION_TYPE_SECURITY_SELL || (trans->type() == TRANSACTION_TYPE_INCOME && ((Income*) trans)->security())) {
+			b_join = false;
+			break;
+		}
+		i = (TransactionListViewItem*) selection.next();
+	}
+	SplitTransaction *split = NULL;
+	i = (TransactionListViewItem*) selection.first();
+	while(i) {
+		Transaction *trans = i->transaction();
+		if(!split) split = trans->parentSplit();
+		if(!trans->parentSplit() || trans->parentSplit() != split) {
+			b_split = false;
+			break;
+		}
+		i = (TransactionListViewItem*) selection.next();
+	}
+	mainWin->ActionNewRefund->setEnabled(refundable);
+	mainWin->ActionNewRepayment->setEnabled(repayable);
+	mainWin->ActionNewRefundRepayment->setEnabled(refundable || repayable);
 	mainWin->ActionEditTransaction->setEnabled(b_transaction);
 	mainWin->ActionDeleteTransaction->setEnabled(b_transaction);
 	mainWin->ActionEditScheduledTransaction->setEnabled(b_scheduledtransaction);
 	mainWin->ActionDeleteScheduledTransaction->setEnabled(b_scheduledtransaction);
+	mainWin->ActionEditSplitTransaction->setEnabled(b_split);
+	mainWin->ActionDeleteSplitTransaction->setEnabled(b_split);
+	mainWin->ActionJoinTransactions->setEnabled(b_join);
+	mainWin->ActionSplitUpTransaction->setEnabled(b_split);
 }
 void TransactionListWidget::filterCategoryActivated(int index) {
 	if(index > 0) editWidget->setCurrentToItem(index - 1);
